%%% readTPTPAxioms
%%%		Action: read axioms from default files

readTPTPAxioms:-
		getTPTPFileList(LF),
		readTPTPAxioms(LF).

%%% readTPTPAxioms(in LF)
%%%		LF: list of input TPTP files (names without extension)

readTPTPAxioms([]).
readTPTPAxioms([F|LF]):-
		print_to_screen('\t\n'),
		print_to_screen('Current file --> '),
		print_to_screen(F),
		atom_concat(F,'.ax',NF),												% Add the extension to the file name
		open(NF, read, S),														% Open the file
		read_next_line(S,L),													% Read the first line of the file
		parseTPTPAxioms(F,S,1,1,L),												% Parse the file
		close(S),																% Close the file
		!,
		readTPTPAxioms(LF).														% Read the rest of files

%%% parseTPTPAxioms(in F, in S, in C, in I, in L)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of axioms
%%%		I: index of the first line of the axiom
%%%		L: first line of the current axiom

parseTPTPAxioms(F,S,C,I,L):-
		readTPTPAxiom(F,S,I,L,IFL,ICL,NT,A,NL),									% Read axiom
		parseTPTPAxiom(F,S,C,IFL,ICL,NT,A,NL).									% Parse axiom

%%% readTPTPAxiom(in F, in S, in I, in L, out IFL, out ICL, out NT, out A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of lines
%%%		L: first line of the next axiom
%%%		IFL: index of the first line of the axiom
%%%		ICL: index of the last line of the axiom
%%%		NT: text of the axiom
%%%		A: the axiom
%%%		NL: new first line of the next axiom

readTPTPAxiom(F,S,I,L,IFL,ICL,NT,A,NL):-
/*
(
 (
  I = 39,
  !,
  trace
 );
 (
  true
 )
),
*/
		(
		 (
		  1 is I mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t\t'),
		  print_to_screen('Current line --> '),
		  print_to_screen(I)
		 );
		 (
		  1 is I mod 10,
		  !,
		  II is I mod 100,
		  print_to_screen(' '),
		  print_to_screen(II)
		 );
		 (
		  true
		 )
		),
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  IFL = I,																% Return the index of the first line (any)
		  ICL = I,																% Return the index of the last line (any)
		  NT = [],																% Return the text of the axiom
		  A = end_of_file,														% Return the axiom (end_of_file)
		  NL = []																% Return the first line of the next axiom (empty line)
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveTPTPAxiom(F,I,I,text,text,text,text,true),						% Store the line
		  read_next_line(S,RL),													% Read the next line
		  readTPTPAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)								% Recursive call to "readTPTPAxiom/9"
		 );
		 (
		  L = [C|_],															% Get the first characters
		  char_code('%',C),														% Comment
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveTPTPAxiom(F,I,I,text,text,text,text,true),						% Store the line
		  read_next_line(S,RL),													% Read the next line
		  readTPTPAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)								% Recursive call to "readTPTPAxiom/9"
		 );
		 (
		  readTPTPTailAxiom(F,S,I,[L],L,NI,NT,A,NL),							% Call to "readTPTPTailAxiom/9"
		  !,
		  IFL = I,																% Return the index of the first line of the axiom
		  ICL = NI																% Return the index of the last line of the axiom
		 );
		 (
		  print_to_screen('\n'),
		  print_to_screen('Incorrect axiom at line: '),							% The axiom is not correct
		  print_to_screen(I),
		  print_to_screen('\n'),
		  fail
		 )
		).

%%% readTPTPTailAxiom(in F, in S, in I, in T, in PL, out NI, out NT, out, A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of the current line of the current axiom
%%%		T: text of the axiom that has been read
%%%		PL: previous lines of the current axiom
%%%		NI: index of the last line of the axiom
%%%		NT: text of axioms
%%%		A: the axiom
%%%		NL: new first line of the next axiom

readTPTPTailAxiom(F,S,I,T,PL,NI,NT,A,NL):-
		read_next_line(S,L),													% Read the next line
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code('%',C),														% Comment
		  !,
		  RI is I + 1,															% New index of the current line of the axiom
		  (
		   (
		    1 is RI mod 100,
		    !,
		    print_to_screen('\n'),
		    print_to_screen('\t\t'),
		    print_to_screen('Current line --> '),
		    print_to_screen(RI)
		   );
		   (
		    1 is RI mod 10,
		    !,
		    NRI is RI mod 100,
		    print_to_screen(' '),
		    print_to_screen(NRI)
		   );
		   (
		    true
		   )
		  ),
		  readTPTPTailAxiom(F,S,RI,T,PL,NI,NT,A,NL)								% Recursive call to "readTPTPTailAxiom/9"
		 );
		 (
		  RI is I + 1,															% New index of the current line of the axiom
		  append(T,[L],RT),														% Current text of the axiom
		  append(PL,L,NPL),														% Current axiom
		  (
		   (
		    1 is RI mod 100,
		    !,
		    print_to_screen('\n'),
		    print_to_screen('\t\t'),
		    print_to_screen('Current line --> '),
		    print_to_screen(RI)
		   );
		   (
		    1 is RI mod 10,
		    !,
		    NRI is RI mod 100,
		    print_to_screen(' '),
		    print_to_screen(NRI)
		   );
		   (
		    true
		   )
		  ),
		  readTPTPTailAxiom(F,S,RI,RT,NPL,NI,NT,A,NL)							% Recursive call to "readTPTPTailAxiom/9"
		 )
		).

%%% parseTPTPAxiom(in F, in S, in C, in IFL, in ICL, in T, in A, in NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of axioms
%%%		IFL: index of the first line of the axiom
%%%		ICL: index of the current line of the current axiom
%%%		T: text of the axiom
%%%		A: the axiom
%%%		NL: first line of the next axiom

parseTPTPAxiom(F,_,C,_,L,_,end_of_file,_):-
		!,
		NC is C - 1,
		setNumberOfTPTPAxioms(F,NC),
		NL is L - 1,
		setNumberOfTPTPLines(F,NL).
parseTPTPAxiom(F,S,C,IFL,ICL,_,A,L):-
%trace,
/*
(
 (
  IFL = 1843,
  !,
  trace
 );
 (
  true
 )
),
*/
		tptpFormula(N,R,NF,A,[]),												% Parse the axiom
		!,
		getAxiomKey(F,C,K),														% Assign a key to the axiom
		getAxiomType(NF,T),
		saveTPTPAxiom(F,IFL,ICL,N,R,K,T,NF),									% Store the axiom
		NC is C + 1,															% New index of axioms
		NICL is ICL + 1,														% New index of lines
		parseTPTPAxioms(F,S,NC,NICL,L).											% Parse the rest of the file
parseTPTPAxiom(_,_,_,IFL,ICL,T,_,_):-
		print_to_screen('Incorrect axiom in lines '),
		print_to_screen(IFL),
		print_to_screen('-'),
		print_to_screen(ICL),
		print_to_screen('\n'),
		print_text_axiom(T),
		fail.

getAxiomType(F,unitClause):-
		isUnitClause(F),
		!.
getAxiomType(_,nonAtomicAxiom).

% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
% %						% %
% %		The parser		% %
% %						% %
% % % % % % % % % % % % % %
% % % % % % % % % % % % % %

% % Rules for tptpFormula % %

tptpFormula(N,R,F) --> nextToken, "fof(", tptpFormulaName(N), nextToken, ",",
						tptpFormulaRole(R), nextToken, ",", tptpFOFormula(F), nextToken, ")", nextToken, ".".
tptpFormula(N,R,F) --> nextToken, "cnf(", tptpFormulaName(N), nextToken, ",",
						tptpFormulaRole(R), nextToken, ",", tptpFOFormula(F), nextToken, ")", nextToken, ".".

% % Rules for tptpFormulaName % %

tptpFormulaName(N) --> tptpAtomicWord(N), !.
tptpFormulaName(N) --> tptpInteger(N).

% % Rules for tptpFormulaRole % %

tptpFormulaRole(R) --> nextToken, "axiom", !,
				{ R = axiom }.
tptpFormulaRole(R) --> nextToken, "conjecture", !,
				{ R = conjecture }.
tptpFormulaRole(R) --> nextToken, "negated_conjecture",
				{ R = negated_conjecture }.

% % Rules for tptpFOFormula % %

tptpFOFormula(NF) --> tptpFOUnitaryFormula(F), tptpFOTailBinaryFormula(F,NF).

% % Rules for tptpFOTailBinaryFormula % %

tptpFOTailBinaryFormula(F,NF) --> tptpBinaryConnective(C), !, tptpFOFormula(BF),
					{ ( ( C = reverseImplies, !, NF =..[implies,BF,F] ) ; ( ( C = equiv; C = implies), !, NF =..[C,F,BF] ) ; ( NF =..[C,[F,BF]] ) ) }.
tptpFOTailBinaryFormula(F,F) --> "".

% % Rules for tptpBinaryConnective % %

tptpBinaryConnective(R) --> nextToken, "&", !,
				{ R = and }.
tptpBinaryConnective(R) --> nextToken, "|", !,
				{ R = or }.
tptpBinaryConnective(R) --> nextToken, "<=>", !,
				{ R = equiv }.
tptpBinaryConnective(R) --> nextToken, "=>", !,
				{ R = implies }.
tptpBinaryConnective(R) --> nextToken, "<=",
				{ R = reverseImplies }.

% % Rules for tptpFOUnitaryFormula % %

tptpFOUnitaryFormula(F) --> nextToken, "(", tptpFOFormula(F), nextToken, ")", !.
tptpFOUnitaryFormula(F) --> tptpFOQuantifiedFormula(F), !.
tptpFOUnitaryFormula(F) --> tptpFOUnaryFormula(F), !.
%tptpFOUnitaryFormula(F) --> tptpFOAtomicFormula(F), !.

% % Rules for tptpFOQuantifiedFormula % %

tptpFOQuantifiedFormula(NF) --> tptpFOQuantifier(Q), nextToken, "[", tptpFOVariableList(LV), nextToken, "]",
						nextToken, ":", tptpFOUnitaryFormula(F),				%%% Instead of tptpFOUnitaryFormula(F), we could use tptpFOFormula(F)
				{ NF =..[Q,LV,F] }.

% % Rules for tptpFOQuantifier % %

tptpFOQuantifier(Q) --> nextToken, "!", !,
				{ Q = 'forall*' }.
tptpFOQuantifier(Q) --> nextToken, "?", !,
				{ Q = 'exists*' }.

% % Rules for tptpFOVariableList % %

tptpFOVariableList([V|LV]) --> tptpFOVariable(V), tptpFOTailVariableList(V,LV).

% % Rules for tptpFOTailVariableList % %

tptpFOTailVariableList(V,[V|LV]) --> nextToken, "," ,!, tptpFOVariableList(LV).
tptpFOTailVariableList(V,[V]) --> "".

% % Rules for tptpFOVariable % %

tptpFOVariable(V) --> tptpUpperWord(N),
				{ V =..['#var',N] }.

% % Rules for tptpFOUnaryFormula % %

tptpFOUnaryFormula(NF) --> tptpUnaryConnective(C), !, tptpFOUnitaryFormula(F),
				{ NF =..[C,F] }.
tptpFOUnaryFormula(F) --> tptpFOInfixUnaryFormula(F).

% % Rules for tptpUnaryConnective % %

tptpUnaryConnective(C) --> nextToken, "~",
				{ C = not }.

% % Rules for tptpFOInfixUnaryFormula % %

tptpFOInfixUnaryFormula(NF) --> tptpFOTerm(T), tptpFOTailInfixUnaryFormula(T,NF).

% % Rules for tptpFOTailInfixUnaryFormula % %

tptpFOTailInfixUnaryFormula(T1,NF) --> tptpInfixUnaryConnective(C), !, tptpFOTerm(T2),
				{ F = equal(T1,T2), ( ( C = equal, !, NF = F ) ; ( NF = not(F) ) ) }.
tptpFOTailInfixUnaryFormula(F,F) --> "".

% % Rules for tptpFOInfixUnaryConnective % %

tptpInfixUnaryConnective(_) --> nextToken, "=>", !,
				{ fail }.
tptpInfixUnaryConnective(C) --> nextToken, "=", !,
				{ C = equal }.
tptpInfixUnaryConnective(C) --> nextToken, "!=", 
				{ C = notEqual }.

% % Rules for tptpFOAtomicFormula % %

tptpFOAtomicFormula(F) --> tptpPredicate(P), nextToken, "(", tptpArgumentList(LT), nextToken, ")",
				{ F =..[P|LT] }.

% % Rules for tptpArgumentList % %

tptpArgumentList(LT) --> tptpFOTerm(T), tptpTailArgumentList(T,LT).

% % Rules for tptpTailArgumentList % %

tptpTailArgumentList(T,[T|LT]) --> nextToken, "," ,!, tptpArgumentList(LT).
tptpTailArgumentList(T,[T]) --> "".

% % Rules for tptpFOTerm % %

tptpFOTerm(T) --> tptpFOFunctionTerm(T), !.
tptpFOTerm(T) --> tptpFOVariable(T).

% % Rules for tptpFOFunctionTerm % %

tptpFOFunctionTerm(T) --> tptpFOPlainTerm(T), !.
tptpFOFunctionTerm(T) --> tptpFODefinedTerm(T), !.
tptpFOFunctionTerm(T) --> tptpFOSystemTerm(T).

% % Rules for tptpFOPlainTerm % %

tptpFOPlainTerm(T) --> tptpFunctor(F), nextToken, "(", tptpArgumentList(LT), nextToken, ")", !,
				{ T =..[F|LT] }.
tptpFOPlainTerm(T) --> tptpConstant(T).

% % Rules for tptpFODefinedTerm % %

tptpFODefinedTerm(T) --> tptpFODefinedAtom(T), !.
%tptpFODefinedTerm(T) --> tptpDefinedAtomicTerm(T).

% % Rules for tptpFOSystemTerm % %

tptpFOSystemTerm(T) --> nextToken, "$", tptpConstant(C),
					{ atom_concat('$',C,T) }.

% % Rules for tptpConstant % %

tptpConstant(T) --> tptpFunctor(T).

% % Rules for tptpFunctor % %

tptpFunctor(F) --> tptpAtomicWord(F).

% % Rules for tptpPredicate % %

tptpPredicate(P) --> tptpAtomicWord(P).

% % Rules for tptpFODefinedAtom % %

tptpFODefinedAtom(A) --> tptpNumber(A), !.
tptpFODefinedAtom(A) --> tptpDistinctObject(A).

% % Rules for tptpNumber % %

tptpNumber(N) --> tptpReal(N), !.
tptpNumber(N) --> tptpRational(N), !.
tptpNumber(N) --> tptpInteger(N).

% % Rules for tptpInteger % %

tptpInteger(I) --> tptpSignedInteger(I), !.
tptpInteger(I) --> tptpUnsignedInteger(I).

% % Rules for tptpSignedInteger % %

tptpSignedInteger(I) --> tptpSign(S), tptpDecimal(D),
					{ atom_concat(S,D,I) }.

% % Rules for tptpUnsignedInteger % %

tptpUnsignedInteger(I) --> tptpDecimal(I).

% % tptpSign % %

tptpSign(S) --> nextToken, "+", !,
					{ S = '+' }.
tptpSign(S) --> nextToken, "-",
					{ S = '-' }.

% % Rules for tptpDecimal % %

tptpDecimal(D) --> listOfNumeric(LC),
					{ LC \= [], atom_codes(D,LC) }.

% % Rules for tptpRational % %

tptpRational(R) --> tptpSignedRational(R), !.
tptpRational(R) --> tptpUnsignedRational(R).

% % Rules for tptpSignedRational % %

tptpSignedRational(SR) --> tptpSign(S), tptpUnsignedRational(R),
					{ atom_concat(S,R,SR) }.

% % Rules for tptpUnsignedRational % %

tptpUnsignedRational(R) --> tptpUnsignedInteger(N), nextToken, "/", tptpUnsignedInteger(D),
					{ atom_concant(N,'/',NN), atom_concat(NN,D,R) }.

% % Rules for tptpReal % %

tptpReal(R) --> tptpInteger(D), nextToken, ".", tptpUnsignedInteger(F), tptpExponent(E),
					{ atom_concat(D,'.',ND), atom_concat(ND,F,N), atom_concat(N,E,R) }.

% % Rules for tptpExponent % %

tptpExponent(E) --> tptpExpSymbol(S), tptpInteger(I), !,
					{ atom_concat(S,I,E) }.
tptpExponent('') --> "".

% % Rules for tptpExpSymbol % %

tptpExpSymbol(E) --> nextToken, "e", !,
					{ E = 'E' }.
tptpExpSymbol(E) --> nextToken, "E",
					{ E = 'E' }.

% % Rules for tptpAtomicWord % %

tptpAtomicWord(N) --> tptpLowerWord(N), !.
tptpAtomicWord(N) --> tptpSingleQuoted(N).

% % Rules for tptpLowerWord % %

tptpLowerWord(N) --> lowerAlpha(C), listOfAlphaNumeric(LC),
				{ atom_codes(N,[C|LC]) }.

% % Rules for tptpUpperWord % %

tptpUpperWord(N) --> upperAlpha(C), listOfAlphaNumeric(LC),
				{ atom_codes(N,[C|LC]) }.

% % Rules for tptpSingleQuoted % %

tptpSingleQuoted(N) --> nextToken, "'", tptpExtendedWord(N), nextToken, "'". 	% Should be <sq_char><sq_char>*

% % Rules for tptpExtendedWord % %

tptpExtendedWord(N) --> listOfExtendedAlphaNumeric(LC),
				{ LC \= [], atom_codes(N,LC) }.

% % Rules for listOfAlphaNumeric % %

listOfAlphaNumeric([C|LC]) --> lowerAlpha(C), !, listOfAlphaNumeric(LC).
listOfAlphaNumeric([C|LC]) --> upperAlpha(C), !, listOfAlphaNumeric(LC).
listOfAlphaNumeric([C|LC]) --> numeric(C), !, listOfAlphaNumeric(LC).
listOfAlphaNumeric([C|LC]) --> wordSymbol(C), !, listOfAlphaNumeric(LC).
listOfAlphaNumeric([]) --> "".

% % Rules for listOfAlphaNumeric % %

listOfExtendedAlphaNumeric([C|LC]) --> lowerAlpha(C), !, listOfExtendedAlphaNumeric(LC).
listOfExtendedAlphaNumeric([C|LC]) --> upperAlpha(C), !, listOfExtendedAlphaNumeric(LC).
listOfExtendedAlphaNumeric([C|LC]) --> numeric(C), !, listOfExtendedAlphaNumeric(LC).
listOfExtendedAlphaNumeric([C|LC]) --> wordSymbol(C), !, listOfExtendedAlphaNumeric(LC).
listOfExtendedAlphaNumeric([C|LC]) --> extraCharacter(C), !, listOfExtendedAlphaNumeric(LC).
listOfExtendedAlphaNumeric([]) --> "".

% % Rules for listOfNumeric % %

listOfNumeric([C|LC]) --> numeric(C), !, listOfNumeric(LC).
listOfNumeric([]) --> "".

% % Rules for lowerAlpha % %

lowerAlpha(C) --> nextToken, [C],
				{ char_code('a',CFirst), char_code('z',CLast), C >= CFirst, C =< CLast }.

% % Rules for upperAlpha % %

upperAlpha(C) --> nextToken, [C],
				{ char_code('A',CFirst), char_code('Z',CLast), C >= CFirst, C =< CLast }.

% % Rules for numeric % %

numeric(D) --> nextToken, [D],
				{ char_code('0',CFirst), char_code('9',CLast), D >= CFirst, D =< CLast }.

% % Rules for wordSymbol % %

wordSymbol(C) --> nextToken, [C],
				{ char_code('_',C) },
				!.

% % Rules for extraCharacter % %

extraCharacter(C) --> nextToken, [C],
				{ char_code('%',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('&',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('|',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('~',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('=',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('>',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('<',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('?',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('!',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(',',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('.',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(':',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(';',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(' ',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('(',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(')',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('{',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('}',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('[',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code(']',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('+',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('-',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('*',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('/',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('^',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('$',C) },
				!.
extraCharacter(C) --> nextToken, [C],
				{ char_code('`',C) },
				!.

% % Rules for tptpDistinctObject % %

tptpDistinctObject(O) --> nextToken, [C], tptpAnyVisibleChar(LC), nextToken, [C], 
					{ char_code('"',C), append(C,LC,CLC), append(CLC,C,NLC), atom_codes(O,NLC) }.

% % Rules for tptpAnyVisibleChar % %

tptpAnyVisibleChar([C|LC]) --> nextToken, [C],
					{ char_code('"',CC), C \= CC },
					!, tptpAnyVisibleChar(LC).
tptpAnyVisibleChar([]).

% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
% %						% %
% %		End of parser	% %
% %						% %
% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
