%%% writeGoals
%%%		Action: write goals in the output files

writeGoals:-
		findall((F,N),numberOfGoals(F,N),LFN),
		catch(make_directory('Goals'),_,true),
		writeGoalsListOfFiles(LFN),
		catch(make_directory('Runs'),_,true),
		writeExecutionScripts.

%%% writeGoalsListOfFiles(in LFN)
%%%		LFN: a list of pairs (F,N) where
%%%			F is the file name
%%%			N is the number of goals
%%%		Action: write goals in the output files

writeGoalsListOfFiles([]).
writeGoalsListOfFiles([(F,N)|LFN]):-
		getAutomaticProver(AP),
		getOutputLanguage(OL),
		writeGoalsFile(F,OL,AP,1,N,RG),
		print_to_screen('\t'),
		print_to_screen('File '),
		print_to_screen(F),
		print_to_screen(' completed (total goals --> '),
		print_to_screen(RG),
		print_to_screen(').'),
		print_to_screen('\n'),
		writeGoalsListOfFiles(LFN).

%%% writeGoalsFile(in F, in OL, in AP, in CG, in NG, out NRG)
%%%		F: a file name
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		IG: index of goals
%%%		NG: total number of goals in the file
%%%		NRG: number of real goals that have been written to the file
%%%		Action: write automatically sorted goals in the output file

writeGoalsFile(_,_,_,IG,NG,0):-
		IG > NG,
		!.
writeGoalsFile(F,OL,AP,IG,NG,NRG):-
		getGoalKey(F,IG,K),
		getGoal(K,G),
		atom_concat(AP,'.',NAP),
		atom_concat(NAP,OL,SOL),
		atom_concat('Goals/',K,K1),
		atom_concat(K1,'.adimen.',K2),
		atom_concat(K2,SOL,NK),
		open(NK,write,S),
		writeGoal(S,OL,AP,G,K,RG1),
		close(S),
		NIG is IG + 1,
		writeGoalsFile(F,OL,AP,NIG,NG,RG2),
		NRG is RG1 + RG2.

%%% writeGoal(in S, in OL, in AP, in G, in K, out NRA)
%%%		S: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		G: goal
%%%		K: goal key
%%%		NRA: number of real goals that have been written to the file
%%%		Action: the goal is written in the output stream

% % TSTP format % %

writeGoal(S,tstp,AP,G,K,1):-
		print_to_file(S,'fof( '),
		print_to_file(S,K),
		print_to_file(S,', conjecture,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tstp,AP,1,G,_),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % TPTP format % %

writeGoal(S,tptp,AP,G,K,1):-
		print_to_file(S,'input_formula( '),
		print_to_file(S,K),
		print_to_file(S,', conjecture,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tptp,AP,1,G,_),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % KIF format % %

writeGoal(S,kif,_,G,_,0):-
		writeFormula(S,0,kif,_,0,G,_),
		!.

% % Error % %

writeGoal(_,L,AP,_,K,_):-
		print_to_screen('Error writting the goal '),
		print_to_screen(K),
		print_to_screen('\n'),
		print_to_screen('Language --> '),
		print_to_screen(L),
		print_to_screen('\n'),
		print_to_screen('Automatic prover --> '),
		print_to_screen(AP),
		print_to_screen('\n'),
		fail.

%%% Input-output predicates %%%

%%% print_text_goal(in T)
%%%		T: the text of an goal

print_text_goal([]).
print_text_goal([LC|T]):-
		atom_codes(L,LC),
		print_to_screen(L),
		print_to_screen('\n'),
		print_text_goal(T).


%%% writeExecutionScripts
%%%		Action:	scripts for the execution of tests are sorted

writeExecutionScripts:-
%		getAutomaticProver(AP),
%		writeExecutionSingleScript(AP),
%		findall(F,numberOfGoals(F,_),LF),
%		writeExecutionScriptsByFile(AP,LF),
%		writeExecutionScriptDefaultCheck(AP),
%		writeExecutionScriptAutomaticCheck(AP).
%trace,
		writeFastTestExecutionScripts('whiteBoxFalsityTest',14,600),
		writeFastTestExecutionScripts('whiteBoxTruthTest',14,600).
%		writeFastTestExecutionScripts('orWhiteBoxFalsityTest',14,600),
%		writeFastTestExecutionScripts('orWhiteBoxTruthTest',14,600).
%		writeFastTestExecutionScripts('multipleMapping',4,36000).


%%% writeFastTestExecutionScripts(in K, in P, in T)
%%%		K: test root key
%%%		P: number of parallel tests
%%%		T: execution-time limit per test
%%%		Action: scripts for the rapid execution of tests are sorted

writeFastTestExecutionScripts(K,P,T):-
		getAutomaticProver(AP),
		numberOfGoals(K,N),
%		NP is round(N/P)+1,
		NP is ceiling(N/P),
		number_chars(T,LC),
		atom_chars(CT,LC),
		writeFastAutomaticTestExecutionScripts(K,1,1,NP,CT,AP).

%%% writeFastAutomaticTestExecutionScripts(in K, in P, in IT, in NT, in T, in AP)
%%%		K: test root key
%%%		P: script number
%%%		IT: number of initial test
%%%		NT: number of tests per script
%%%		T: execution-time limit per test
%%%		AP: the automatic prover to be used
%%%		Action: scripts for the rapid execution of tests are sorted

writeFastAutomaticTestExecutionScripts(K,_,IT,_,_,_):-
		numberOfGoals(K,N),
		IT > N,
		!.

% Vampire (from v8.1)

writeFastAutomaticTestExecutionScripts(K,P,IT,NT,T,vampire):-
		numberOfGoals(K,N),
		IT =< N,
%		LT1 is IT + NT,
		LT1 is IT + NT-1,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.vampire.tstp Axioms/adimen.sumo.vampire.tstp > Runs/',
		SF3301 = '.adimen.sumo.vampire.tstp\nvampire3.0 --proof tptp --output_axiom_names on --mode casc -t ',
		atom_concat(SF3301,T,SF3302),
		atom_concat(SF3302,' Runs/',SF330),
%		SF3261 = '.adimen.sumo.vampire.tstp\n./vampire2.6 --proof tptp --output_axiom_names on --mode casc -t ',
%		atom_concat(SF3261,T,SF3262),
%		atom_concat(SF3262,' Runs/',SF326),
%		SF3181 = '.adimen.sumo.vampire.tstp\n./vampire1.8 --proof tptp --output_axiom_names on --mode casc -t ',
%		atom_concat(SF3181,T,SF3182),
%		atom_concat(SF3182,' Runs/',SF318),
		SF4 = '.adimen.sumo.vampire.tstp > Runs/output.',
		SF530 = '.adimen.sumo.vampire3.0.txt\nrm Runs/',
%		SF526 = '.adimen.sumo.vampire2.6.txt\nrm Runs/',
%		SF518 = '.adimen.sumo.vampire1.8.0.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
%		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF326,SF4,SF526,SF6,C)),LC26),
%		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF318,SF4,SF518,SF6,C)),LC18),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat(FN6,'Vampire3.0.sh',FN30),
		open(FN30,write,S30),
		(
		 (
		  P = 1,
		  !,
		  writeSourceCommand(S30,vampire)
		 );
		 (
		  true
		 )
		),
		writeListOfCommands(S30,LC30),
		writeAlertCommand(S30,'Vampire3.0','FastTests'),
		close(S30),
/*
		atom_concat(FN6,'Vampire2.6.sh',FN26),
		open(FN26,write,S26),
		(
		 (
		  P = 1,
		  !,
		  writeSourceCommand(S26,vampire)
		 );
		 (
		  true
		 )
		),
		writeListOfCommands(S26,LC26),
		writeAlertCommand(S26,'Vampire2.6','FastTests'),
		close(S26),
		atom_concat(FN6,'Vampire1.8.sh',FN18),
		open(FN18,write,S18),
		(
		 (
		  P = 1,
		  !,
		  writeSourceCommand(S18,vampire)
		 );
		 (
		  true
		 )
		),
		writeListOfCommands(S18,LC18),
		writeAlertCommand(S18,'Vampire1.8','FastTests'),
		close(S18),
*/
		NP is P+1,
		NIT is LT+1,
		writeFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,vampire).

% E prover (v1.8)

writeFastAutomaticTestExecutionScripts(K,P,IT,NT,T,eprover):-
		numberOfGoals(K,N),
		IT =< N,
		LT1 is IT + NT,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.eprover.tstp Axioms/adimen.sumo.eprover.tstp > Runs/',
		SF31 = '.adimen.sumo.eprover.tstp\neprover1.8 --auto --memory-limit=4096 --cpu-limit=',
		atom_concat(SF31,T,SF32),
		atom_concat(SF32,' --print-pid --tstp-in -l 6 Runs/',SF3),
		SF4 = '.adimen.sumo.eprover.tstp > Runs/output.',
		SF5 = '.adimen.sumo.eprover.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C)),LC),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat(FN6,'EProver1.8.sh',FN),
		open(FN,write,S),
		(
		 (
		  P = 1,
		  !,
		  writeSourceCommand(S,eprover)
		 );
		 (
		  true
		 )
		),
		writeListOfCommands(S,LC),
		writeAlertCommand(S,'EProver1.8','FastTests'),
		close(S),
		NP is P+1,
		NIT is LT+1,
		writeFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,eprover).

%%% writeExecutionSingleScript(in AP)
%%%		AP: the automatic prover to be used
%%%		Action:	a script for the execution of all tests are sorted

% Vampire (from v8.1)

writeExecutionSingleScript(vampire):-
		SF1 = 'cat Goals/',
		SF2 = '.adimen.vampire.tstp Axioms/adimen.sumo.vampire.tstp > Runs/',
		SF330 = '.adimen.sumo.vampire.tstp\nvampire3.0 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF326 = '.adimen.sumo.vampire.tstp\nvampire2.6 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF318 = '.adimen.sumo.vampire.tstp\nvampire1.8 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF4 = '.adimen.sumo.vampire.tstp > Runs/output.',
		SF530 = '.adimen.sumo.vampire3.0.txt\nrm Runs/',
		SF526 = '.adimen.sumo.vampire2.6.txt\nrm Runs/',
		SF518 = '.adimen.sumo.vampire1.8.0.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(goal(_,_,_,G,_),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
		findall(C,(goal(_,_,_,G,_),composeCommand16(G,SF1,SF2,SF326,SF4,SF526,SF6,C)),LC26),
		findall(C,(goal(_,_,_,G,_),composeCommand16(G,SF1,SF2,SF318,SF4,SF518,SF6,C)),LC18),
		open('Runs/runAllTestsVampire3.0.sh',write,S30),
		writeSourceCommand(S30,vampire),
		writeListOfCommands(S30,LC30),
		writeAlertCommand(S30,'Vampire3.0','AllTests'),
		close(S30),
		open('Runs/runAllTestsVampire2.6.sh',write,S26),
		writeSourceCommand(S26,vampire),
		writeListOfCommands(S26,LC26),
		writeAlertCommand(S26,'Vampire2.6','AllTests'),
		close(S26),
		open('Runs/runAllTestsVampire1.8.sh',write,S18),
		writeSourceCommand(S18,vampire),
		writeListOfCommands(S18,LC18),
		writeAlertCommand(S18,'Vampire1.8','AllTests'),
		close(S18).

% E prover (v1.8)

writeExecutionSingleScript(eprover):-
		SF1 = 'cat Goals/',
		SF2 = '.adimen.eprover.tstp Axioms/adimen.sumo.eprover.tstp > Runs/',
		SF3 = '.adimen.sumo.eprover.tstp\neprover1.8 --auto --memory-limit=4096 --cpu-limit=7200 --print-pid --tstp-in -l 6 Runs/',
		SF4 = '.adimen.sumo.eprover.tstp > Runs/output.',
		SF5 = '.adimen.sumo.eprover.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(goal(_,_,_,G,_),composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C)),LC),
		open('Runs/runAllTestsEProver1.8.sh',write,S),
		writeSourceCommand(S,eprover),
		writeListOfCommands(S,LC),
		writeAlertCommand(S,'EProver1.8','AllTests'),
		close(S).

%%% writeExecutionScriptsByFile(in AP, in LF)
%%%		AP: the automatic prover to be used
%%%		LF: list of files
%%%		Action:	scripts for the execution of tests by file are sorted

writeExecutionScriptsByFile(_,[]).

% Vampire (from v8.1)

writeExecutionScriptsByFile(vampire,[F|LF]):-
		SF1 = 'cat Goals/',
		SF2 ='.adimen.vampire.tstp Axioms/adimen.sumo.vampire.tstp > Runs/',
		SF330 = '.adimen.sumo.vampire.tstp\nvampire3.0 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF326 = '.adimen.sumo.vampire.tstp\nvampire2.6 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF318 = '.adimen.sumo.vampire.tstp\nvampire1.8 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/',
		SF4 = '.adimen.sumo.vampire.tstp > Runs/output.',
		SF530 = '.adimen.sumo.vampire3.0.txt\nrm Runs/',
		SF526 = '.adimen.sumo.vampire2.6.txt\nrm Runs/',
		SF518 = '.adimen.sumo.vampire1.8.0.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(goal(F,_,_,G,_),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
		findall(C,(goal(F,_,_,G,_),composeCommand16(G,SF1,SF2,SF326,SF4,SF526,SF6,C)),LC26),
		findall(C,(goal(F,_,_,G,_),composeCommand16(G,SF1,SF2,SF318,SF4,SF518,SF6,C)),LC18),
		atom_concat('Runs/run',F,OF1),
		atom_concat(OF1,'TestsVampire',OF2),
		atom_concat(OF2,'3.0',OF230),
		atom_concat(OF2,'2.6',OF226),
		atom_concat(OF2,'1.8',OF218),
		atom_concat(OF230,'.sh',NOF30),
		atom_concat(OF226,'.sh',NOF26),
		atom_concat(OF218,'.sh',NOF18),
		open(NOF30,write,S30),
		writeSourceCommand(S30,vampire),
		writeListOfCommands(S30,LC30),
		close(S30),
		open(NOF26,write,S26),
		writeSourceCommand(S26,vampire),
		writeListOfCommands(S26,LC26),
		close(S26),
		open(NOF18,write,S18),
		writeSourceCommand(S18,vampire),
		writeListOfCommands(S18,LC18),
		close(S18),
		writeExecutionScriptsByFile(vampire,LF).

% E prover (v1.8)

writeExecutionScriptsByFile(eprover,[F|LF]):-
		SF1 = 'cat Goals/',
		SF2 ='.adimen.eprover.tstp Axioms/adimen.sumo.eprover.tstp > Runs/',
		SF3 = '.adimen.sumo.eprover.tstp\neprover1.8 --auto --memory-limit=4096 --cpu-limit=7200 --print-pid --tstp-in -l 6 Runs/',
		SF4 = '.adimen.sumo.eprover.tstp > Runs/output.',
		SF5 = '.adimen.sumo.eprover.txt\nrm Runs/',
		SF6 = '.adimen.sumo.vampire.tstp\n\n',
		findall(C,(goal(F,_,_,G,_),composeCommand15(G,SF1,SF2,SF3,SF4,SF5,SF6,C)),LC),
		atom_concat('Runs/run',F,OF1),
		atom_concat(OF1,'TestsEProver1.8.sh',NOF),
		open(NOF,write,S),
		writeSourceCommand(S,eprover),
		writeListOfCommands(S,LC),
		close(S),
		writeExecutionScriptsByFile(eprover,LF).

%%% writeExecutionScriptDefaultCheck(in AP)
%%%		AP: the automatic prover to be used
%%%		Action:	a script for the execution of the default check is created

% Vampire (from v8.1)

writeExecutionScriptDefaultCheck(vampire):-
		open('Runs/runDefaultCheckVampire3.0.sh',write,S30),
		writeSourceCommandDefaultCheck(S30,vampire),
		print_to_file(S30,'vampire3.0 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/DefaultCheck/defaultCheck.adimen.sumo.vampire.tstp > Runs/DefaultCheck/output.defaultCheck.adimen.sumo.vampire3.0.txt\n\n'),
		writeAlertCommand(S30,'Vampire3.0','DefaultCheck'),
		close(S30),
		open('Runs/runDefaultCheckVampire2.6.sh',write,S26),
		writeSourceCommandDefaultCheck(S26,vampire),
		print_to_file(S26,'vampire2.6 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/DefaultCheck/defaultCheck.adimen.sumo.vampire.tstp > Runs/DefaultCheck/output.defaultCheck.adimen.sumo.vampire2.6.txt\n\n'),
		writeAlertCommand(S26,'Vampire2.6','DefaultCheck'),
		close(S26),
		open('Runs/runDefaultCheckVampire1.8.sh',write,S18),
		writeSourceCommandDefaultCheck(S18,vampire),
		print_to_file(S18,'vampire1.8 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/DefaultCheck/defaultCheck.adimen.sumo.vampire.tstp > Runs/DefaultCheck/output.defaultCheck.adimen.sumo.vampire1.8.txt\n\n'),
		writeAlertCommand(S18,'Vampire1.8','DefaultCheck'),
		close(S18).

% E prover (v1.8)

writeExecutionScriptDefaultCheck(eprover):-
		open('Runs/runDefaultCheckEProver1.8.sh',write,S),
		writeSourceCommandDefaultCheck(S,eprover),
		print_to_file(S,'eprover1.8 --auto --memory-limit=8192 --cpu-limit=1382400 --print-pid --tstp-in -l 6 Runs/DefaultCheck/defaultCheck.adimen.sumo.eprover.tstp > Runs/DefaultCheck/output.defaultCheck.adimen.sumo.eprover.txt\n\n'),
		writeAlertCommand(S,'EProver1.8','DefaultCheck'),
		close(S).

%%% writeExecutionScriptAutomaticCheck(in AP)
%%%		AP: the automatic prover to be used
%%%		Action:	a script for the execution of the automatic check is created

% Vampire (from v8.1)

writeExecutionScriptAutomaticCheck(vampire):-
		open('Runs/runAutomaticCheckVampire3.0.sh',write,S30),
		writeSourceCommandAutomaticCheck(S30,vampire),
		print_to_file(S30,'vampire3.0 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/AutomaticCheck/automaticCheck.adimen.sumo.vampire.tstp > Runs/AutomaticCheck/output.automaticCheck.adimen.sumo.vampire3.0.txt\n\n'),
		writeAlertCommand(S30,'Vampire3.0','AutomaticCheck'),
		close(S30),
		open('Runs/runAutomaticCheckVampire2.6.sh',write,S26),
		writeSourceCommandAutomaticCheck(S26,vampire),
		print_to_file(S26,'vampire2.6 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/AutomaticCheck/automaticCheck.adimen.sumo.vampire.tstp > Runs/AutomaticCheck/output.automaticCheck.adimen.sumo.vampire2.6.txt\n\n'),
		writeAlertCommand(S26,'Vampire2.6','AutomaticCheck'),
		close(S26),
		open('Runs/runAutomaticCheckVampire1.8.sh',write,S18),
		writeSourceCommandAutomaticCheck(S18,vampire),
		print_to_file(S18,'vampire1.8 --proof tptp --output_axiom_names on --mode casc -t 36000 Runs/AutomaticCheck/automaticCheck.adimen.sumo.vampire.tstp > Runs/AutomaticCheck/output.automaticCheck.adimen.sumo.vampire1.8.txt\n\n'),
		writeAlertCommand(S18,'Vampire1.8','AutomaticCheck'),
		close(S18).

% E prover (v1.8)

writeExecutionScriptAutomaticCheck(eprover):-
		open('Runs/runAutomaticCheckEProver1.8.sh',write,S),
		writeSourceCommandAutomaticCheck(S,eprover),
		print_to_file(S,'eprover1.8 --auto --memory-limit=8192 --cpu-limit=1382400 --print-pid --tstp-in -l 6 Runs/AutomaticCheck/automaticCheck.adimen.sumo.eprover.tstp > Runs/AutomaticCheck/output.automaticCheck.adimen.sumo.eprover.txt\n\n'),
		writeAlertCommand(S,'EProver1.8','AutomaticCheck'),
		close(S).

%%% writeListOfCommands(in S, in LC)
%%%		S: output stream
%%%		LC: list of commands
%%%		Action: the list of commands is written in S

writeListOfCommands(_,[]).
writeListOfCommands(S,[C|LC]):-
		print_to_file(S,C),
		writeListOfCommands(S,LC).

%%% writeSourceCommand(in S, in AP)
%%%		S: output stream
%%%		AP: the automatic prover to be used
%%%		Action: the command to obtain the source file is written in S

writeSourceCommand(S,AP):-
		findall(F,(numberOfAxioms(F,_),F \= 'automaticCheck',F \= 'whiteBox',F \= 'orWhiteBox'),LF),
		print_to_file(S,'cat'),
		writeSourceCommandListOfFiles(S,AP,LF),
		print_to_file(S,' > '),
		atom_concat('Axioms/adimen.sumo.',AP,TF1),
		atom_concat(TF1,'.',TF2),
		getOutputLanguage(OL),
		atom_concat(TF2,OL,TF),
		print_to_file(S,TF),
		print_to_file(S,'\n\n').

%%% writeSourceCommandDefaultCheck(in S, in AP)
%%%		S: output stream
%%%		AP: the automatic prover to be used
%%%		Action: the command to obtain the source file is written in S

writeSourceCommandDefaultCheck(S,AP):-
		findall(F,(numberOfAxioms(F,_),F \= 'automaticCheck',F \= 'whiteBox',F \= 'orWhiteBox'),LF),
		print_to_file(S,'cat'),
		writeSourceCommandListOfFiles(S,AP,LF),
		print_to_file(S,' > '),
		atom_concat('Runs/DefaultCheck/defaultCheck.adimen.sumo.',AP,TF1),
		atom_concat(TF1,'.',TF2),
		getOutputLanguage(OL),
		atom_concat(TF2,OL,TF),
		print_to_file(S,TF),
		print_to_file(S,'\n\n').

%%% writeSourceCommandAutomaticCheck(in S, in AP)
%%%		S: output stream
%%%		AP: the automatic prover to be used
%%%		Action: the command to obtain the source file is written in S

writeSourceCommandAutomaticCheck(S,AP):-
		findall(F,(numberOfAxioms(F,_),F \= 'automaticCheck',F \= 'whiteBox',F \= 'orWhiteBox'),LF),
		print_to_file(S,'cat'),
		writeSourceCommandListOfFiles(S,AP,LF),
		print_to_file(S,' > '),
		atom_concat('Runs/AutomaticCheck/automaticCheck.adimen.sumo.',AP,TF1),
		atom_concat(TF1,'.',TF2),
		getOutputLanguage(OL),
		atom_concat(TF2,OL,TF),
		print_to_file(S,TF),
		print_to_file(S,'\n\n').

%%% writeSourceCommandListOfFiles(in S, in AP, in LF)
%%%		S: output stream
%%%		AP: the automatic prover to be used
%%%		LF: list of files
%%%		Action: the command to obtain the source file is written in S

writeSourceCommandListOfFiles(_,_,[]).
writeSourceCommandListOfFiles(S,AP,[F|LF]):-
		getOutputLanguage(OL),
		atom_concat(AP,'.',NAP),
		atom_concat(NAP,OL,SOL),
		atom_concat('Axioms/',F,F1),
		atom_concat(F1,'.adimen.',F2),
		atom_concat(F2,SOL,NF),
		print_to_file(S,' '),
		print_to_file(S,NF),
		writeSourceCommandListOfFiles(S,AP,LF).

%%% writeAlertCommand(in S, in AP, in T)
%%%		S: output stream
%%%		AP: the automatic prover used
%%%		T: the kind of test performed
%%%		Action: the command to obtain the source file is written in S

writeAlertCommand(S,AP,T):-
		print_to_file(S,'php Runs/'),
		print_to_file(S,AP),
		print_to_file(S,'.'),
		print_to_file(S,T),
		print_to_file(S,'.mail.php'),
		print_to_file(S,'\n').

%%% composeCommand16(in G, in SF1, in SF2, in SF3, in SF4, in SF5, in SF6, out C)
%%%		G: goal name
%%%		SF1, SF2, SF3, SF4, SF5, SF6: command portions
%%%		C: command

composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C):-
		atom_concat(SF1,G,SFG1),
		atom_concat(SF2,G,SFG2),
		atom_concat(SF3,G,SFG3),
		atom_concat(SF4,G,SFG4),
		atom_concat(G,SF6,SFG6),
		atom_concat(SFG1,SFG2,SC1),
		atom_concat(SC1,SFG3,SC2),
		atom_concat(SC2,SFG4,SC3),
		atom_concat(SC3,SF5,SC4),
		atom_concat(SC4,SFG6,C).
