%%% writeTPTPAxioms
%%%		Action: write TPTP axioms in the output files

writeTPTPAxioms:-
		findall((F,C),(getTPTPFileList(L),numberOfTPTPAxioms(F,C),member(F,L)),LFC),
		catch(make_directory('Axioms'),_,true),
		writeTPTPAxiomsListOfFiles(LFC).

%%% writeTPTPAxiomsListOfFiles(in LFL)
%%%		LFL: a list of pairs (F,L) where
%%%			F is the file name
%%%			L is the number of lines
%%%		Action: write axioms in the output files

writeTPTPAxiomsListOfFiles([]).
writeTPTPAxiomsListOfFiles([(F,L)|LFL]):-
		getAutomaticProver(AS),
		getOutputLanguage(OL),
		atom_concat(AS,'.',NAS),
		atom_concat(NAS,OL,SOL),
		atom_concat('Axioms/',F,F2),
		atom_concat(F2,'.adimen.',F3),
		atom_concat(F3,SOL,NF),
		open(NF,write,S),
		writeTPTPAxiomsFile(F,S,OL,AS,1,L,RA,UC),
		close(S),
		print_to_screen('\t'),
		print_to_screen('File '),
		print_to_screen(NF),
		print_to_screen(' completed (total axioms --> '),
		print_to_screen(RA),
		print_to_screen(', unit clauses --> '),
		print_to_screen(UC),
		print_to_screen(').'),
		print_to_screen('\n'),
		writeAxiomsListOfFiles(LFL).



%%% writeTPTPAxiomsFile(in F, in S, in OL, in AP, in CL, in TL, out NRA, out NUC)
%%%		F: a file name
%%%		S: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		CL: current line of the file
%%%		TL: total number of lines in the file
%%%		NRA: number of real axioms that have been written to the file
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: write automatically sorted axioms in the output file

writeTPTPAxiomsFile(_,_,_,_,CL,TL,0,0):-
		CL > TL,
		!.
writeTPTPAxiomsFile(F,S,OL,AP,CL,TL,NRA,NUC):-
		getAxiomKey(F,CL,K),
		getTPTPAxiom(K,_,A),
		writeAxiomWithoutText(S,OL,AP,0,A,K,RA1,UC1),
		NCL is CL + 1,
		writeTPTPAxiomsFile(F,S,OL,AP,NCL,TL,RA2,UC2),
		NRA is RA1 + RA2,
		NUC is UC1 + UC2.
