%%% writeTPTPGoals
%%%		Action: write goals in the output files

writeTPTPGoals:-
		findall((F,N),numberOfTPTPGoals(F,N),LFN),
		catch(make_directory('Goals'),_,true),
		writeTPTPGoalsListOfFiles(LFN),
		catch(make_directory('Runs'),_,true),
		writeTPTPExecutionScripts.

%%% writeTPTPGoalsListOfFiles(in LFN)
%%%		LFN: a list of pairs (F,N) where
%%%			F is the file name
%%%			N is the number of goals
%%%		Action: write goals in the output files

writeTPTPGoalsListOfFiles([]).
writeTPTPGoalsListOfFiles([(F,N)|LFN]):-
		getAutomaticProver(AP),
		getOutputLanguage(OL),
		writeTPTPGoalsFile(F,OL,AP,1,N,RG),
		print_to_screen('\t'),
		print_to_screen('File '),
		print_to_screen(F),
		print_to_screen(' completed (total goals --> '),
		print_to_screen(RG),
		print_to_screen(').'),
		print_to_screen('\n'),
		writeTPTPGoalsListOfFiles(LFN).

%%% writeTPTPGoalsFile(in F, in OL, in AP, in CG, in NG, out NRG)
%%%		F: a file name
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		IG: index of goals
%%%		NG: total number of goals in the file
%%%		NRG: number of real goals that have been written to the file
%%%		Action: write automatically sorted goals in the output file

writeTPTPGoalsFile(_,_,_,IG,NG,0):-
		IG > NG,
		!.
writeTPTPGoalsFile(F,OL,AP,IG,NG,NRG):-
		getGoalKey(F,IG,K),
		getTPTPGoal(K,G),
		atom_concat(AP,'.',NAP),
		atom_concat(NAP,OL,SOL),
		atom_concat('Goals/',K,K1),
		atom_concat(K1,'.adimen.',K2),
		atom_concat(K2,SOL,NK),
		open(NK,write,S),
		writeTPTPGoal(S,OL,AP,G,K,RG1),
		close(S),
		NIG is IG + 1,
		writeTPTPGoalsFile(F,OL,AP,NIG,NG,RG2),
		NRG is RG1 + RG2.

%%% writeTPTPGoal(in S, in OL, in AP, in G, in K, out NRA)
%%%		S: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		G: goal
%%%		K: goal key
%%%		NRA: number of real goals that have been written to the file
%%%		Action: the goal is written in the output stream

% % TSTP format % %

writeTPTPGoal(S,tstp,AP,G,K,1):-
		print_to_file(S,'fof( '),
		print_to_file(S,K),
		print_to_file(S,', conjecture,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tstp,AP,1,G,_),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

%%% writeTPTPExecutionScripts
%%%		Action:	scripts for the execution of tests are sorted

writeTPTPExecutionScripts:-
%		getAutomaticProver(AP),
%		writeExecutionSingleScript(AP),
%		findall(F,numberOfGoals(F,_),LF),
%		writeExecutionScriptsByFile(AP,LF),
%		writeExecutionScriptDefaultCheck(AP),
%		writeExecutionScriptAutomaticCheck(AP).
%trace,
		writeTPTPFastTestExecutionScripts('whiteBoxFalsityTest',20,600),
		writeTPTPFastTestExecutionScripts('whiteBoxTruthTest',10,600).
%		writeTPTPFastTestExecutionScripts('orWhiteBoxFalsityTest',10,600),
%		writeTPTPFastTestExecutionScripts('orWhiteBoxTruthTest',5,600).
%		writeFastTestExecutionScripts('multipleMapping',4,36000).


%%% writeTPTPFastTestExecutionScripts(in K, in P, in T)
%%%		K: test root key
%%%		P: number of parallel tests
%%%		T: execution-time limit per test
%%%		Action: scripts for the rapid execution of tests are sorted

writeTPTPFastTestExecutionScripts(K,P,T):-
		getAutomaticProver(AP),
		numberOfTPTPGoals(K,N),
%		NP is round(N/P)+1,
		NP is ceiling(N/P),
		number_chars(T,LC),
		atom_chars(CT,LC),
		writeTPTPFastAutomaticTestExecutionScripts(K,1,1,NP,CT,AP).

%%% writeTPTPFastAutomaticTestExecutionScripts(in K, in P, in IT, in NT, in T, in AP)
%%%		K: test root key
%%%		P: script number
%%%		IT: number of initial test
%%%		NT: number of tests per script
%%%		T: execution-time limit per test
%%%		AP: the automatic prover to be used
%%%		Action: scripts for the rapid execution of tests are sorted

writeTPTPFastAutomaticTestExecutionScripts(K,_,IT,_,_,_):-
		numberOfTPTPGoals(K,N),
		IT > N,
		!.

% Vampire (from v8.1)

writeTPTPFastAutomaticTestExecutionScripts(K,P,IT,NT,T,vampire):-
		numberOfTPTPGoals(K,N),
		IT =< N,
%		LT1 is IT + NT,
		LT1 is IT + NT-1,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.vampire.tstp Axioms/DolceCASL.ax > Runs/',
		SF3301 = '.adimen.CSR.vampire.tstp\nvampire4.1 --proof tptp --output_axiom_names on --mode casc -t ',
		atom_concat(SF3301,T,SF3302),
		atom_concat(SF3302,' Runs/',SF330),
		SF4 = '.adimen.CSR.vampire.tstp > Runs/output.',
		SF530 = '.adimen.CSR.vampire4.1.txt\nrm Runs/',
		SF6 = '.adimen.CSR.vampire.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat(FN6,'Vampire4.1.sh',FN30),
		open(FN30,write,S30),
		writeListOfCommands(S30,LC30),
		writeAlertCommand(S30,'Vampire4.1','FastTests'),
		close(S30),
		NP is P+1,
		NIT is LT+1,
		writeTPTPFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,vampire).

% E prover (v1.8)

writeTPTPFastAutomaticTestExecutionScripts(K,P,IT,NT,T,eprover):-
		numberOfTPTPGoals(K,N),
		IT =< N,
		LT1 is IT + NT,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.eprover.tstp Axioms/adimen.CSR.eprover.tstp > Runs/',
		SF31 = '.adimen.CSR.eprover.tstp\neprover1.8 --auto --memory-limit=4096 --cpu-limit=',
		atom_concat(SF31,T,SF32),
		atom_concat(SF32,' --print-pid --tstp-in -l 6 Runs/',SF3),
		SF4 = '.adimen.CSR.eprover.tstp > Runs/output.',
		SF5 = '.adimen.CSR.eprover.txt\nrm Runs/',
		SF6 = '.adimen.CSR.vampire.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C)),LC),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat(FN6,'EProver1.8.sh',FN),
		open(FN,write,S),
		writeListOfCommands(S,LC),
		writeAlertCommand(S,'EProver1.8','FastTests'),
		close(S),
		NP is P+1,
		NIT is LT+1,
		writeTPTPFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,eprover).
