% %
% %	createTPTPGoals
% %
% %		Goals are created automatically

createTPTPGoals:-
		findall((F,C),(getTPTPFileList(L),numberOfTPTPAxioms(F,C),member(F,L)),LFC),
		open('whiteBox.txt',write,S),
%leash([]),trace,
		createTPTPGoalsListOfFiles(S,0,0,LFC),
		close(S).

% %
% %	createTPTPOrGoals
% %
% %		Or goals are created automatically

createTPTPOrGoals:-
		findall((F,C),(getTPTPFileList(L),numberOfTPTPAxioms(F,C),member(F,L)),LFC),
		open('orWhiteBox.txt',write,S),
%leash([]),trace,
		createTPTPOrGoalsListOfFiles(S,0,0,LFC),
		close(S).

%%% createTPTPGoalsListOfFiles(in S, in I, in T, in LFC)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		Action: goals are created automatically

createTPTPGoalsListOfFiles(_,I,T,[]):-
		setNumberOfTPTPGoals('whiteBoxFalsityTest',I),
		setNumberOfTPTPGoals('whiteBoxTruthTest',I),
		setNumberOfTPTPLines('whiteBox',I),
		setNumberOfTPTPAxioms('whiteBox',I),
		setNumberOfTPTPTests(T).
createTPTPGoalsListOfFiles(S,I,T,[(F,C)|LFC]):-
		createTPTPGoalsFile(S,I,T,F,0,C,NI,NT),
		createTPTPGoalsListOfFiles(S,NI,NT,LFC).

%%% createTPTPOrGoalsListOfFiles(in S, in I, in T, in LFC)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		Action: or goals are created automatically

createTPTPOrGoalsListOfFiles(_,I,T,[]):-
		setNumberOfTPTPGoals('orWhiteBoxFalsityTest',I),
		setNumberOfTPTPGoals('orWhiteBoxTruthTest',I),
		setNumberOfTPTPLines('orWhiteBox',I),
		setNumberOfTPTPAxioms('orWhiteBox',I),
		setNumberOfTPTPTests(T).
createTPTPOrGoalsListOfFiles(S,I,T,[(F,C)|LFC]):-
		createTPTPOrGoalsFile(S,I,T,F,0,C,NI,NT),
		createTPTPOrGoalsListOfFiles(S,NI,NT,LFC).

%%% createTPTPGoalsFile(in S, in G, in T, in F, in I, in N, out NG, out NT)
%%%		S: output stream
%%%		G: index of goals
%%%		T: index of tests
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		NG: new index of goals
%%%		NT: new index of tests
%%%		Action: goals are created automatically

createTPTPGoalsFile(_,G,T,_,N,N,G,T):-
		!.
createTPTPGoalsFile(S,G,T,F,I,N,NG,NT):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
%print_to_screen('\n'),
%print_to_screen(K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
/*
(
 (
  K = 'CSR002+1-v6.3.0A18',
  !,
  leash([]), trace
 );
 (
  notrace
 )
),
*/
		getTPTPAxiom(K,_,A),
		correctType(A,A1),
		distributeQuantifiers(A1,A2),
		compactAxiom(A2,A3),
		simplifyAxiom(A3,1,A4),
		compactAxiom(A4,SF),
		createTPTPAxiomGoals(S,G,T,K,SF,G1,T1),
		createTPTPGoalsFile(S,G1,T1,F,NI,N,NG,NT).

%%% createTPTPOrGoalsFile(in S, in G, in T, in F, in I, in N, out NG, out NT)
%%%		S: output stream
%%%		G: index of goals
%%%		T: index of tests
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		NG: new index of goals
%%%		NT: new index of tests
%%%		Action: or goals are created automatically

createTPTPOrGoalsFile(_,G,T,_,N,N,G,T):-
		!.
createTPTPOrGoalsFile(S,G,T,F,I,N,NG,NT):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
%print_to_screen('\n'),
%print_to_screen(K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
/*
(
 (
  K = 'milo1133A1188',
  !,
  leash([]), trace
 );
 (
  notrace
 )
),
*/
		getTPTPAxiom(K,_,A),
		createTPTPAxiomOrGoals(S,G,T,K,A,G1,T1),
		createTPTPOrGoalsFile(S,G1,T1,F,NI,N,NG,NT).

%%% createTPTPAxiomGoals(in S, in I, in T, in K, in A, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tets
%%%		K: an axiom key
%%%		A: an axiom
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from A

createTPTPAxiomGoals(S,I,T,K,A,NI,NT):-
		createTPTPFormulaGoals(S,I,T,K,[],A,NI,NT).

%%% createTPTPAxiomOrGoals(in S, in I, in T, in K, in A, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tets
%%%		K: an axiom key
%%%		A: an axiom
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from A

createTPTPAxiomOrGoals(S,I,T,K,A,NI,NT):-
		createTPTPFormulaOrGoals(S,I,T,K,[],A,NI,NT).

%%% createTPTPFormulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createTPTPFormulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPFormulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPFormulaGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createTPTPFormulaGoals(S,I,T,K,LV,F,NI,NT).
createTPTPFormulaGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createTPTPFormulaGoals(S,I,T,K,LV,F,I1,T1),
		createTPTPFormulaGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createTPTPFormulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPFormulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPFormulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createTPTPFormulaGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPFormulaGoals(S,I,T,K,LV,F,NI,NT):-
		createTPTPSubformulaGoals(S,I,T,K,LV,F,NI,NT).

%%% createTPTPNonClosedFormulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createTPTPNonClosedFormulaGoals(S,I,T,K,LV,F,NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createTPTPNonClosedFormulaGoals(S,I,T,K,LV,F,I1,T1),
		createTPTPNonClosedFormulaGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createTPTPNonClosedFormulaGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPNonClosedFormulaGoals(S,I,T,K,LV,F,NI,NT):-
		createTPTPNonClosedSubformulaGoals(S,I,T,K,LV,F,NI,NT).

%%% createTPTPFormulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createTPTPFormulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createTPTPFormulaOrGoals(S,I,T,K,LV,F,NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createTPTPFormulaOrGoals(S,I,T,K,LV,F,I1,T1),
		createTPTPFormulaOrGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPFormulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createTPTPFormulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPFormulaOrGoals(S,I,T,K,LV,F,NI,NT):-
		createTPTPSubformulaOrGoals(S,I,T,K,LV,F,NI,NT).

%%% createTPTPNonClosedFormulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,LV,F,NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,LV,F,I1,T1),

		createTPTPNonClosedFormulaOrGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createTPTPNonClosedFormulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPNonClosedFormulaOrGoals(S,I,T,K,LV,F,NI,NT):-
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LV,F,NI,NT).

%%% createTPTPSubformulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createTPTPSubformulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		saveTPTPTest(S,I,K,forallTest,'forall*'(LVP,'forall*'(LV,F)),I1),
		saveTPTPTest(S,I1,K,negatedForallTest,'forall*'(LVP,not('forall*'(LV,F))),I2),
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaGoals(S,I2,T,K,NLVP,F,NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		saveTPTPTest(S,I,K,existsTest,'forall*'(LVP,'exists*'(LV,F)),I1),
		saveTPTPTest(S,I1,K,negatedExistsTest,'forall*'(LVP,not('exists*'(LV,F))),I2),
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaGoals(S,I2,T,K,NLVP,F,NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createTPTPAndSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createTPTPOrSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveTPTPTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1),
		saveTPTPTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2),
		saveTPTPTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3),
		saveTPTPTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4),
		createTPTPSubformulaGoals(S,I4,T,K,LVP,not(F1),I5,T1),
		createTPTPSubformulaGoals(S,I5,T1,K,LVP,F2,NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPSubformulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPSubformulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createTPTPSubformulaGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPSubformulaGoals(_,I,T,_,_,_,I,T).

%%% createTPTPNonClosedSubformulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		saveTPTPNonClosedTest(S,I,K,forallTest,'forall*'(LVP,'forall*'(LV,F)),I1,TKG1),
		saveTPTPNonClosedTest(S,I1,K,negatedForallTest,'forall*'(LVP,not('forall*'(LV,F))),I2,TKG2),
		append(LVP,LV,NLVP),
		T1 is T+1,
		saveTPTPTestInformation(K,forallTest,T1,TKG1,TKG2),
		createTPTPNonClosedSubformulaGoals(S,I2,T1,K,NLVP,F,NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		saveTPTPNonClosedTest(S,I,K,existsTest,'forall*'(LVP,'exists*'(LV,F)),I1,TKG1),
		saveTPTPNonClosedTest(S,I1,K,negatedExistsTest,'forall*'(LVP,not('exists*'(LV,F))),I2,TKG2),
		append(LVP,LV,NLVP),
		T1 is T+1,
		saveTPTPTestInformation(K,existsTest,T1,TKG1,TKG2),
		createTPTPNonClosedSubformulaGoals(S,I2,T1,K,NLVP,F,NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createTPTPNonClosedAndSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createTPTPNonClosedOrSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveTPTPNonClosedTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1,TKG1),
		saveTPTPNonClosedTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2,TKG2),
		saveTPTPNonClosedTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3,TKG3),
		saveTPTPNonClosedTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4,TKG4),
		T1 is T+1,
		saveTPTPTestInformation(K,antecedentImpliesTest,T1,TKG1,TKG2),
		T2 is T1+1,
		saveTPTPTestInformation(K,consequentImpliesTest,T2,TKG3,TKG4),
		createTPTPNonClosedSubformulaGoals(S,I4,T2,K,LVP,not(F1),I5,T3),
		createTPTPNonClosedSubformulaGoals(S,I5,T3,K,LVP,F2,NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createTPTPNonClosedSubformulaGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPNonClosedSubformulaGoals(_,I,T,_,_,_,I,T).

%%% createTPTPSubformulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createTPTPSubformulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createTPTPAndSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createTPTPOrSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveTPTPOrTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1),
		saveTPTPOrTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2),
		saveTPTPOrTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3),
		saveTPTPOrTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4),
		createTPTPSubformulaOrGoals(S,I4,T,K,LVP,not(F1),I5,T1),
		createTPTPSubformulaOrGoals(S,I5,T1,K,LVP,F2,NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPSubformulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPSubformulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createTPTPSubformulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPSubformulaOrGoals(_,I,T,_,_,_,I,T).

%%% createTPTPNonClosedSubformulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createTPTPNonClosedAndSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createTPTPNonClosedOrSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveTPTPNonClosedOrTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1,TKG1),
		saveTPTPNonClosedOrTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2,TKG2),
		saveTPTPNonClosedOrTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3,TKG3),
		saveTPTPNonClosedOrTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4,TKG4),
		T1 is T+1,
		saveTPTPTestInformation(K,antecedentImpliesTest,T1,TKG1,TKG2),
		T2 is T1+1,
		saveTPTPTestInformation(K,consequentImpliesTest,T2,TKG3,TKG4),
		createTPTPNonClosedSubformulaOrGoals(S,I4,T2,K,LVP,not(F1),I5,T3),
		createTPTPNonClosedSubformulaOrGoals(S,I5,T3,K,LVP,F2,NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createTPTPNonClosedSubformulaOrGoals(_,I,T,_,_,_,I,T).

%%% createTPTPAndSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from and(LF)

createTPTPAndSubformulasGoals(_,I,T,_,_,[],I,T).
createTPTPAndSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPTest(S,I,K,andTest,'forall*'(LVP,F),I1),
		saveTPTPTest(S,I1,K,negatedAndTest,'forall*'(LVP,not(F)),I2),
		createTPTPSubformulaGoals(S,I2,T,K,LVP,F,I3,T1),
		createTPTPAndSubformulasGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createTPTPNonClosedAndSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from and(LF)

createTPTPNonClosedAndSubformulasGoals(_,I,T,_,_,[],I,T).
createTPTPNonClosedAndSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPNonClosedTest(S,I,K,andTest,'forall*'(LVP,F),I1,TKG1),
		saveTPTPNonClosedTest(S,I1,K,negatedAndTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTPTPTestInformation(K,andTest,T1,TKG1,TKG2),
		createTPTPNonClosedSubformulaGoals(S,I2,T1,K,LVP,F,I3,T2),
		createTPTPNonClosedAndSubformulasGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% createTPTPAndSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from and(LF)

createTPTPAndSubformulasOrGoals(_,I,T,_,_,[],I,T).
createTPTPAndSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		createTPTPSubformulaOrGoals(S,I,T,K,LVP,F,I1,T1),
		createTPTPAndSubformulasOrGoals(S,I1,T1,K,LVP,LF,NI,NT).

%%% createTPTPNonClosedAndSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from and(LF)

createTPTPNonClosedAndSubformulasOrGoals(_,I,T,_,_,[],I,T).
createTPTPNonClosedAndSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		createTPTPNonClosedSubformulaOrGoals(S,I,T,K,LVP,F,I1,T1),
		createTPTPNonClosedAndSubformulasOrGoals(S,I1,T1,K,LVP,LF,NI,NT).

%%% createTPTPOrSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from or(LF)

createTPTPOrSubformulasGoals(_,I,T,_,_,[],I,T).
createTPTPOrSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPTest(S,I,K,orTest,'forall*'(LVP,F),I1),
		saveTPTPTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2),
		createTPTPSubformulaGoals(S,I2,T,K,LVP,F,I3,T1),
		createTPTPOrSubformulasGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createTPTPNonClosedOrSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from or(LF)

createTPTPNonClosedOrSubformulasGoals(_,I,T,_,_,[],I,T).
createTPTPNonClosedOrSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPNonClosedTest(S,I,K,orTest,'forall*'(LVP,F),I1,TKG1),
		saveTPTPNonClosedTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTPTPTestInformation(K,andTest,T1,TKG1,TKG2),
		createTPTPNonClosedSubformulaGoals(S,I2,T1,K,LVP,F,I3,T2),
		createTPTPNonClosedOrSubformulasGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% createTPTPOrSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from or(LF)

createTPTPOrSubformulasOrGoals(_,I,T,_,_,[],I,T).
createTPTPOrSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPOrTest(S,I,K,orTest,'forall*'(LVP,F),I1),
		saveTPTPOrTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2),
		createTPTPSubformulaOrGoals(S,I2,T,K,LVP,F,I3,T1),
		createTPTPOrSubformulasOrGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createTPTPNonClosedOrSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from or(LF)

createTPTPNonClosedOrSubformulasOrGoals(_,I,T,_,_,[],I,T).
createTPTPNonClosedOrSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTPTPNonClosedOrTest(S,I,K,orTest,'forall*'(LVP,F),I1,TKG1),
		saveTPTPNonClosedOrTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTPTPTestInformation(K,andTest,T1,TKG1,TKG2),
		createTPTPNonClosedSubformulaOrGoals(S,I2,T1,K,LVP,F,I3,T2),
		createTPTPNonClosedOrSubformulasOrGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% saveTPTPTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: a test
%%%		NI: new index of goals
%%%		TKG: key of the new truth-test
%%%		Action: the new test is stored

saveTPTPTest(S,I,K,M,T,I):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getTPTPGoalFile(FKG,NFST,'whiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getTPTPGoalFile(TKG,NTST,'whiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTPTPTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTPTPTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveTPTPTest(S,I,K,M,T,NI):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		NI is I+1,
		getGoalKey('whiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('whiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveTPTPGoal('whiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTPTPTestMode(FKG,K,M),
		saveTPTPGoal('whiteBoxTruthTest',NI,NI,TKG,TST),
		saveTPTPTestMode(TKG,K,M),
		getAxiomKey('whiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getAxiomType(TST,AT),
		saveTPTPAxiom('whiteBox',NI,NI,KA,axiom,KA,AT,TST).

%%% saveTPTPNonClosedTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: a test
%%%		NI: new index of goals
%%%		TKG: key of the new truth-test
%%%		Action: the new test is stored

saveTPTPNonClosedTest(S,I,K,M,T,I,TKG):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getTPTPGoalFile(FKG,NFST,'whiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getTPTPGoalFile(TKG,NTST,'whiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTPTPTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTPTPTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),

		print_to_file(S,'\n').
saveTPTPNonClosedTest(S,I,K,M,T,NI,TKG):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		NI is I+1,
		getGoalKey('whiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('whiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveTPTPGoal('whiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTPTPTestMode(FKG,K,M),
		saveTPTPGoal('whiteBoxTruthTest',NI,NI,TKG,TST),
		saveTPTPTestMode(TKG,K,M),
		getAxiomKey('whiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getAxiomType(TST,AT),
		saveTPTPAxiom('whiteBox',NI,NI,KA,axiom,KA,AT,TST).

%%% saveTPTPOrTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: an or test
%%%		NI: new index of goals
%%%		TKG: key of the new or truth-test
%%%		Action: the new or test is stored

saveTPTPOrTest(S,I,K,M,T,I):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getTPTPGoalFile(FKG,NFST,'orWhiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getTPTPGoalFile(TKG,NTST,'orWhiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTPTPTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTPTPTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveTPTPOrTest(S,I,K,M,T,NI):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		NI is I+1,
		getGoalKey('orWhiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('orWhiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveTPTPGoal('orWhiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTPTPTestMode(FKG,K,M),
		saveTPTPGoal('orWhiteBoxTruthTest',NI,NI,TKG,TST),
		saveTPTPTestMode(TKG,K,M),
		getAxiomKey('orWhiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getAxiomType(TST,AT),
		saveTPTPAxiom('orWhiteBox',NI,NI,KA,axiom,KA,AT,TST).

%%% saveTPTPNonClosedOrTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: an or test
%%%		NI: new index of goals
%%%		TKG: key of the new or truth-test
%%%		Action: the new or test is stored

saveTPTPNonClosedOrTest(S,I,K,M,T,I,TKG):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getTPTPGoalFile(FKG,NFST,'orWhiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getTPTPGoalFile(TKG,NTST,'orWhiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTPTPTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTPTPTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveTPTPNonClosedOrTest(S,I,K,M,T,NI,TKG):-
		simplifyAxiom(T,FST),
		simplifyAxiom(not(FST),TST),
		NI is I+1,
		getGoalKey('orWhiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('orWhiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveTPTPGoal('orWhiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTPTPTestMode(FKG,K,M),
		saveTPTPGoal('orWhiteBoxTruthTest',NI,NI,TKG,TST),
		saveTPTPTestMode(TKG,K,M),
		getAxiomKey('orWhiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getAxiomType(TST,AT),
		saveTPTPAxiom('orWhiteBox',NI,NI,KA,axiom,KA,AT,TST).

%%% saveTPTPTestMode(in KG, in KA, in M)
%%%		KG: a goal Key
%%%		KA: an axiom key
%%%		M: a test mode
%%%		Action: the test mode is stored

saveTPTPTestMode(KG,KA,M):-
		assertz((tptpGoalInformation(KG,KA,M))).

%%% saveTPTPTestInformation(in K, in M, in I, in TKG1, in TKG2))
%%%		K: an axiom key
%%%		M: a test mode
%%%		I: index of test
%%%		TKG1,TKG2: truth-test keys
%%%		Action: the test information is stored

saveTPTPTestInformation(K,M,T,TKG1,TKG2):-
		assertz((tptpTestInformation(K,M,T,TKG1,TKG2))).

%%% formulaVariablesToPrologVariables(in F, out LPV, out NF)
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing formula variables with
%%%				Prolog variables in F

formulaVariablesToPrologVariables(F,NLPV,NF):-
		toPrologVariables(F,LPV1,F1),
		findall(V,(sub_term(V,F1),isSingleVariable(V),ground(V)),LV),
		replaceListOfFormulaVariables(LV,F1,LPV2,NF),
		append(LPV1,LPV2,NLPV).

%%% toPrologVariables(in F, out LPV, out NF)
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing formula variables with
%%%				Prolog variables in F

toPrologVariables('exists*'(LV,F),NLPV,'exists*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('forall*'(LV,F),NLPV,'forall*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('exists*'(LV,F),NLPV,'exists*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('forall*'(LV,F),NLPV,'forall*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(and(LF),LPV,and(NLF)):-
		!,
		toPrologVariablesListOfFormulas(LF,LPV,NLF).
toPrologVariables(or(LF),LPV,or(NLF)):-
		!,
		toPrologVariablesListOfFormulas(LF,LPV,NLF).
toPrologVariables(implies(F1,F2),NLPV,implies(NF1,NF2)):-
		!,
		toPrologVariables(F1,LPV1,NF1),
		toPrologVariables(F2,LPV2,NF2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(equiv(F1,F2),NLPV,equiv(NF1,NF2)):-
		!,
		toPrologVariables(F1,LPV1,NF1),
		toPrologVariables(F2,LPV2,NF2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(not(F),LPV,not(NF)):-
		!,
		toPrologVariables(F,LPV,NF).
toPrologVariables(F,[],F).

%%% toPrologVariablesListOfFormulas(in LF, out LPV, out NLF)
%%%		LF: a list of formulas
%%%		LPV: a list of Prolog variables
%%%		NLF: the list of formulas that is obtained by replacing formula
%%%				variables with Prolog variables in LF

toPrologVariablesListOfFormulas([],[],[]).
toPrologVariablesListOfFormulas([F|LF],NLPV,[NF|NLF]):-
		toPrologVariables(F,LPV1,NF),
		toPrologVariablesListOfFormulas(LF,LPV2,NLF),
		append(LPV1,LPV2,NLPV).

%%% replaceListOfFormulaVariables(in LV, in F, out LPV, out NF)
%%%		LV: list of variables
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing LV by LPV in F

replaceListOfFormulaVariables([],F,[],F).
replaceListOfFormulaVariables([V|LV],F,[PV|LPV],NF):-
		replaceFormulaVariable(V,PV,F,F1),
		replaceListOfFormulaVariables(LV,F1,LPV,NF).

%%% replaceFormulaVariable(in V, in PV, in F, out NF)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		F: a formula
%%%		NF: the formula that is obtained by replacing V by PV in F

replaceFormulaVariable(V,PV,'exists*'(LV,F),'exists*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'forall*'(LV,F),'forall*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'exists*'(LV,F),'exists*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'forall*'(LV,F),'forall*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,and(LF),and(NLF)):-
		!,
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).
replaceFormulaVariable(V,PV,or(LF),or(NLF)):-
		!,
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).
replaceFormulaVariable(V,PV,implies(F1,F2),implies(NF1,NF2)):-
		!,
		replaceFormulaVariable(V,PV,F1,NF1),
		replaceFormulaVariable(V,PV,F2,NF2).
replaceFormulaVariable(V,PV,equiv(F1,F2),equiv(NF1,NF2)):-
		!,
		replaceFormulaVariable(V,PV,F1,NF1),
		replaceFormulaVariable(V,PV,F2,NF2).
replaceFormulaVariable(V,PV,not(F),not(NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,A,NA):-
		replaceFormulaVariableAtom(V,PV,A,NA).

%%% replaceFormulaVariable(in V, in PV, in F, out NF)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		F: a formula
%%%		NF: the formula that is obtained by replacing V by PV in F

replaceFormulaVariableListOfFormulas(_,_,[],[]).
replaceFormulaVariableListOfFormulas(V,PV,[F|LF],[NF|NLF]):-
		replaceFormulaVariable(V,PV,F,NF),
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).

%%% replaceFormulaVariableAtom(in V, in PV, in A, out NA)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		A: an atom
%%%		NA: the atom that is obtained by replacing V by PV in A

replaceFormulaVariableAtom(_,_,A,A):-
		var(A),
		!.
replaceFormulaVariableAtom(V,PV,V,PV):-
		!.
replaceFormulaVariableAtom(V,PV,A,NA):-
		A =..[P|ST],
		replaceFormulaVariableSubterms(V,PV,ST,NST),
		NA =..[P|NST].

%%% replaceFormulaVariableSubterms(in V, in PV, in LST, out NLST)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		LST: a list of subterms
%%%		NLST: the list of subterms that is obtained by replacing V by PV in ST

replaceFormulaVariableSubterms(_,_,[],[]).
replaceFormulaVariableSubterms(V,PV,[ST|LST],[NST|NLST]):-
		replaceFormulaVariableAtom(V,PV,ST,NST),
		replaceFormulaVariableSubterms(V,PV,LST,NLST).

%%% correctType(in F, out NF)
%%%		F: formula
%%%		NF: the formula that results from distributing implication over double implication

correctType('forall*'(LV,implies(F1,equiv(F2,F3))),and(['forall*'(LV,implies(and([F1,F2]),F3)),'forall*'(LV,implies(and([F1,F3]),F2))])):-
		!.
correctType(F,F).
