toMin:-
		LF = ['Runs/output.whiteBoxFalsityTest5733.adimen.sumo.vampire3.0.txt'], parseListOfProofFiles(LF), createMinimalProofs(LF), halt.

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Part :		Proof parsing												% %
% %																				% %
% %				All the predicates to parse proof								% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Proof Internal Representation											% %
% %																				% %
% %		Each proof step is represented using the following dynamic predicate:	% %
% %																				% %
% %		step(ProofKey,StepKey,Formula,Source)									% %
% %																				% %
% %			ProofKey: it is usually the conjecture name							% %
% %																				% %
% %			StepKey: as provided by the ATP										% %
% %																				% %
% %			Formula: as provided by the ATP										% %
% %																				% %
% %			Source: two options													% %
% %					axiom(AxiomKey) -> if the formula is an input axiom			% %
% %					inference([ListOfStepKeys]) -> if the formula is derived	% %
% %					new -> if the formula is introduced by the ATP				% %
% %																				% %
% %		Almost redundantly, we also use:										% %
% %																				% %
% %		stepAxiom(ProofSubkey,ProofKey,StepKey,AxiomKey,ProofType)				% %
% %																				% %
% %			ProofSubkey: it is usually a prefix of the conjecture name			% %
% %																				% %
% %			ProofKey: it is usually the conjecture name							% %
% %																				% %
% %			StepKey: as provided by the ATP										% %
% %																				% %
% %			AxiomKey: name of the input axioms									% %
% %																				% %
% %			ProofType: 0 or 1, depending on whether the conjecture is used (0)	% %
% %						or not used (1) in the proof							% %
% %																				% %
% %		Redundantly, we also use:												% %
% %																				% %
% %		proofAxioms(ProofKey,ListOfAxiomKeys,Length)							% %
% %																				% %
% %			ProofKey: it is usually the conjecture name							% %
% %																				% %
% %			ListOfAxiomKeys: list of names of the input axioms					% %
% %																				% %
% %			Length: length of ListOfAxiomKeys									% %
% %																				% %
% %		Additionally, we include external information from the ontology:		% %
% %																				% %
% %		axiom(AxiomSubkey,AxiomKey,Formula,Type)								% %
% %																				% %
% %			AxiomSubkey: it is usually a prefix of the axiom name				% %
% %																				% %
% %			AxiomKey: name of the axiom											% %
% %																				% %
% %			Formula: the formula in the axiom									% %
% %																				% %
% %			Type: 'unitClause' or 'nonAtomicAxiom' depending on whether 		% %
% %						the formula is atomic or not							% %
% %																				% %
% %		Finally, we store the proof time:										% %
% %																				% %
% %		proofTime(ProofSubkey,ProofKey,Time)									% %
% %																				% %
% %			ProofSubkey: it is usually a prefix of the conjecture name			% %
% %																				% %
% %			ProofKey: it is usually the conjecture name							% %
% %																				% %
% %			Time: as provided by the ATP										% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

%:- op(500, fx, user:(~)).

:-	dynamic	step/4, stepAxiom/5, proofAxioms/3, axiom/4, proofTime/3.

%%% Delete previous information

deletePreviousInformation:-
		retractall((step(_,_,_,_))),
		retractall((stepAxiom(_,_,_,_,_))),
		retractall((proofAxioms(_,_,_))),
		retractall((axiom(_,_,_,_))),
		retractall((proofTime(_,_,_))).

%%% CSR information

:- consult('AdimenCSRAxioms.pl').

%%% Test information

:- consult('AdimenSUMOTests.pl').


%%% Proof input information

:-	dynamic	proofDirectory/1, minimalProofDirectory/1, minimalProofFile/1.
:-	dynamic	proofFilePrefix/1, proofFileSuffix/1, problemFileSuffix/1.
:-	dynamic	proofSubkey/1.

proofDirectory('Runs/').

minimalProofDirectory('Minimal/').

minimalProofFile('MinimalProofInformation.pl').

proofFilePrefix('output.').

proofFileSuffix('.adimen.CSR.vampire3.0.txt').
%proofFileSuffix('.adimen.tptpSUMO.vampire3.0.txt').

problemFileSuffix('.adimen.CSR.vampire.tstp').
%problemFileSuffix('.adimen.tptpSUMO.tstp').

/*
problemSubkeys([
				'orWhiteBoxFalsityTest'
]).

truthProblemSubkeys([
]).

falsityProblemSubkeys([
				'orWhiteBoxFalsityTest'
]).
*/

/*
problemSubkeys([
				'orWhiteBoxTruthTest',
				'orWhiteBoxFalsityTest'
]).

truthProblemSubkeys([
				'orWhiteBoxTruthTest'
]).

falsityProblemSubkeys([
				'orWhiteBoxFalsityTest'
]).
*/

%/*
problemSubkeys([
				'whiteBoxTruthTest',
				'whiteBoxFalsityTest'
]).

truthProblemSubkeys([
				'whiteBoxTruthTest'
]).

falsityProblemSubkeys([
				'whiteBoxFalsityTest'
]).
%*/

/*
problemSubkeys([
				'compatibleMultipleMapping',
				'incompatibleMultipleMapping'
]).
*/

/*
problemSubkeys([
				'antonym',
				'relation',
				'equalEvent',
				'subclassEvent',
				'orSubclassEvent',
				'negatedAntonym',
				'negatedRelation',
				'negatedEqualEvent',
				'negatedSubclassEvent',
				'negatedOrSubclassEvent'
]).


truthProblemSubkeys([
				'antonym',
				'relation',
				'equalEvent',
				'subclassEvent',
				'orSubclassEvent'
]).

falsityProblemSubkeys([
				'negatedAntonym',
				'negatedRelation',
				'negatedEqualEvent',
				'negatedSubclassEvent',
				'negatedOrSubclassEvent'
]).
*/

%%% getProofDirectory(out D)
%%%		D: output directory

getProofDirectory(D):-
		proofDirectory(D).

%%% setProofDirectory(in D)
%%%		D: output directory
%%%		Action: the output directory is updated

setProofDirectory(D):-
		retractall((proofDirectory(_))),
		assertz((proofDirectory(D))).

%%% getMinimalProofDirectory(out D)
%%%		D: output directory

getMinimalProofDirectory(D):-
		minimalProofDirectory(D).

%%% setMinimalProofDirectory(in D)
%%%		D: output directory
%%%		Action: the output directory for minimal proofs is updated

setMinimalProofDirectory(D):-
		retractall((minimalProofDirectory(_))),
		assertz((minimalProofDirectory(D))).

%%% getMinimalProofFile(out F)
%%%		F: output file

getMinimalProofFile(F):-
		minimalProofFile(F).

%%% setMinimalProofFile(in F)
%%%		F: output file
%%%		Action: the output file for minimal proofs is updated

setMinimalProofFile(F):-
		retractall((minimalProofFile(_))),
		assertz((minimalProofFile(F))).

%%% getProofFilePrefix(out P)
%%%		P: proof file prefix

getProofFilePrefix(P):-
		proofFilePrefix(P).

%%% setProofFilePrefix(in P)
%%%		D: proof file prefix
%%%		Action: the proof file prefix is updated

setProofFilePrefix(P):-
		retractall((proofFilePrefix(_))),
		assertz((proofFilePrefix(P))).

%%% getProofFileSuffix(out P)
%%%		P: proof file suffix

getProofFileSuffix(P):-
		proofFileSuffix(P).

%%% setProofFileSuffix(in P)
%%%		D: proof file suffix
%%%		Action: the proof file suffix is updated

setProofFileSuffix(P):-
		retractall((proofFileSuffix(_))),
		assertz((proofFileSuffix(P))).

%%% getProblemFileSuffix(out P)
%%%		P: problem file suffix

getProblemFileSuffix(P):-
		problemFileSuffix(P).

%%% setProblemFileSuffix(in P)
%%%		D: problem file suffix
%%%		Action: the problem file suffix is updated

setProblemFileSuffix(P):-
		retractall((problemFileSuffix(_))),
		assertz((problemFileSuffix(P))).

%%% getProblemSubkeys(out LSK)
%%%		LSK: list of proof subkeys

getProblemSubkeys(LSK):-
		problemSubkeys(LSK).

%%% setProblemSubkeys(in LSK)
%%%		LSK: list fo problem subkeys
%%%		Action: the list of problem subkeys is updated

setProblemSubkeys(LSK):-
		retractall((problemSubkeys(_))),
		assertz((problemSubkeys(LSK))).

%%% getTruthProblemSubkeys(out LSK)
%%%		LSK: list of truth proof subkeys

getTruthProblemSubkeys(LSK):-
		truthProblemSubkeys(LSK).

%%% setTruthProblemSubkeys(in LSK)
%%%		LSK: list fo truth problem subkeys
%%%		Action: the list of truth problem subkeys is updated

setTruthProblemSubkeys(LSK):-
		retractall((truthProblemSubkeys(_))),
		assertz((truthProblemSubkeys(LSK))).

%%% getFalsityProblemSubkeys(out LSK)
%%%		LSK: list of falsity proof subkeys

getFalsityProblemSubkeys(LSK):-
		falsityProblemSubkeys(LSK).

%%% setFalsityProblemSubkeys(in LSK)
%%%		LSK: list fo falsity problem subkeys
%%%		Action: the list of falsity problem subkeys is updated

setFalsityProblemSubkeys(LSK):-
		retractall((falsityProblemSubkeys(_))),
		assertz((falsityProblemSubkeys(LSK))).





%%% getMinimalProofs

getMinimalProofs:-

		deletePreviousInformation,

		catch(make_directory('Minimal'),_,true),								% Create the directory for minimal proofs, if not existing

		getProblemSubkeys(LSK),

		findall(_,(member(SK,LSK),getListOfProofFiles(SK,LF),parseListOfProofFiles(LF),createMinimalProofs(LF)),_).



%%% getMinimalFalsityTestProofs

getMinimalFalsityTestProofs:-

		deletePreviousInformation,

		catch(make_directory('Minimal'),_,true),								% Create the directory for minimal proofs, if not existing

		getFalsityProblemSubkeys(LSK),

		findall(_,(member(SK,LSK),getListOfProofFiles(SK,LF),parseListOfProofFiles(LF),createMinimalProofs(LF)),_).

%%% getMinimalTruthTestProofs

getMinimalTruthTestProofs:-

		deletePreviousInformation,

		catch(make_directory('Minimal'),_,true),								% Create the directory for minimal proofs, if not existing

		getTruthProblemSubkeys(LSK),

		findall(_,(member(SK,LSK),getListOfProofFiles(SK,LF),parseListOfProofFiles(LF),createMinimalProofs(LF)),_).



%%% getProofsTime

getProofsTime:-

		catch(make_directory('Minimal'),_,true),								% Create the directory for minimal proofs, if not existing

		getProblemSubkeys(LSK),

		findall(_,(member(SK,LSK),getListOfProofFiles(SK,LF),member(F,LF),getProofKey(F,PK),saveProofTime(SK,PK,F)),_).






%%% compileMinimalProofs

compileMinimalProofs:-

		deletePreviousInformation,

		getListOfMinimalProofFiles(LF),											% Get list of minimal proofs

		parseListOfProofFiles(LF),												% Parse minimal proofs

		getListOfProofKeys(LPK),												% Get list of proof keys

		classifyProofs(LPK),													% Classify proofs

		getProofAxioms(LPK),													% Get axioms of the proof

		saveMinimalProofs.														% Save minimal proofs

%%% loadMinimalProofs

loadMinimalProofs:-
		getMinimalProofDirectory(D),
		getMinimalProofFile(F),
		atom_concat(D,F,NF),
		open(NF,read,S),
		loadMinimalProofInformation(S),
		close(S).


%%% For the demo

listAllProofs:-
		findall(PK,stepAxiom(_,PK,_,_,_),LPK),
		list_to_set(LPK,SPK),
		current_output(S),
		print_list(S,SPK).

listAllProofs(SK):-
		findall(PK,stepAxiom(SK,PK,_,_,_),LPK),
		list_to_set(LPK,SPK),
		current_output(S),
		print_list(S,SPK).


print_list(S,[A,B,C]):-
		!,
		print_tabs(S,1),
		print_to_screen(S,A),
		print_tabs(S,1),
		print_to_screen(B),
		print_tabs(S,1),
		print_to_screen(C).
print_list(S,[A,B]):-
		!,
		print_tabs(S,1),
		print_to_screen(A),
		print_tabs(S,1),
		print_to_screen(B).
print_list(S,[A]):-
		!,
		print_tabs(S,1),
		print_to_screen(A).
print_list(_,[]):-
		!.
print_list(S,[A,B,C,D|L]):-
		!,
		print_tabs(S,1),
		print_to_screen(A),
		print_tabs(S,1),
		print_to_screen(B),
		print_tabs(S,1),
		print_to_screen(C),
		print_tabs(S,1),
		print_to_screen(D),
		print_to_screen('\n'),
		print_list(S,L).


getExecutionTimeLimit(600).


%%% numberOfProofs(out N)
%%%		N: the total number of solved problems

numberOfProofs(N):-
		numberOfProofs(_,N).

%%% numberOfProofs(in T, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		N: the number of solved problems of type T

numberOfProofs(T,N):-
		findall(PK,stepAxiom(_,PK,_,_,T),LPK),
		list_to_set(LPK,SPK),
		length(SPK,N).

%%% numberOfProofsOfProblem(in SK, out N)
%%%		SK: problem subkey
%%%		N: the number of proofs of problem SK

numberOfProofsOfProblem(SK,N):-
		numberOfProofsOfProblem(_,SK,N).

%%% numberOfProofsOfProblem(in T,in SK, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		SK: problem subkey
%%%		N: the number of proofs of problem SK

numberOfProofsOfProblem(T,SK,N):-
		findall(PK,stepAxiom(SK,PK,_,_,T),LPK),
		list_to_set(LPK,SPK),
		length(SPK,N).

%%% numberOfProofsOfListOfProblems(in LSK, out N)
%%%		LSK: list of problem subkeys
%%%		N: the number of proofs of problem in LSK

numberOfProofsOfListOfProblems(LSK,N):-
		numberOfProofsOfListOfProblems(_,LSK,N).

%%% numberOfProofsOfListOfProblems(in T,in LSK, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		N: the number of proofs of problem in LSK

numberOfProofsOfListOfProblems(T,LSK,N):-
		findall(PK,(stepAxiom(SK,PK,_,_,T),memberchk(SK,LSK)),LPK),
		list_to_set(LPK,SPK),
		length(SPK,N).

%%% numberOfProofsByProblem(out N)
%%%		N: the number of proofs by problem

numberOfProofsByProblem(N):-
		numberOfProofsByProblem(_,N).

%%% numberOfProofsByProblem(in T, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		N: the number of proofs by problem

numberOfProofsByProblem(T,LSKN):-
		getProblemSubkeys(LSK),
		findall((SK,N),(member(SK,LSK),numberOfProofsOfProblem(T,SK,N)),LSKN).

%%% numberOfProofsByListOfProblems(in LSK, out N)
%%%		LSK: list of problem subkeys
%%%		N: the number of proofs of problem in LSK

numberOfProofsByListOfProblems(LSK,N):-
		numberOfProofsByListOfProblems(_,LSK,N).

%%% numberOfProofsByListOfProblems(in T,in LSK, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		N: the number of proofs of problem in LSK

numberOfProofsByListOfProblems(T,LSK,LSKN):-
		findall((SK,N),(member(SK,LSK),numberOfProofsOfProblem(T,SK,N)),LSKN).

%%% numberOfAxiomsInProofs(out N)
%%%		N: the total number of axioms used in proofs

numberOfAxiomsInProofs(N):-
		numberOfAxiomsInProofs(_,N).

%%% numberOfAxiomsInProofs(in T, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		N: the number of axioms used in proofs

numberOfAxiomsInProofs(T,N):-
		findall(SK,(stepAxiom(_,PK,_,SK,T), PK \= SK), LSK),
		list_to_set(LSK,SSK),
		length(SSK,N).

%%% numberOfAxiomsInProofsOfProblem(in SK, out N)
%%%		SK: problem subkey
%%%		N: the number of axioms used in proofs

numberOfAxiomsInProofsOfProblem(SK,N):-
		numberOfAxiomsInProofsOfProblem(_,SK,N).

%%% numberOfAxiomsInProofsOfProblem(in T, in SK, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		SK: problem subkey
%%%		N: the number of axioms used in proofs

numberOfAxiomsInProofsOfProblem(T,SK,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T), PK \= AK), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

%%% numberOfAxiomsInProofsOfListOfProblems(in LSK, out N)
%%%		LSK: list of problem subkeys
%%%		N: the number of axioms used in proofs

numberOfAxiomsInProofsOfListOfProblems(LSK,N):-
		numberOfAxiomsInProofsOfListOfProblems(_,LSK,N).

%%% numberOfAxiomsInProofsOfListOfProblems(in T, in LSK, out N)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		N: the number of axioms used in proofs

numberOfAxiomsInProofsOfListOfProblems(T,LSK,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

%%% numberOfAxiomsInProofsByProblem(out LSKN)
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByProblem(LSKN):-
		numberOfAxiomsInProofsByProblem(_,LSKN).

%%% numberOfAxiomsInProofsByProblem(in T, out LSKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByProblem(T,LSKN):-
		getProblemSubkeys(LSK),
		findall((SK,N),(member(SK,LSK),numberOfAxiomsInProofsOfProblem(T,SK,N)),LSKN).

%%% numberOfAxiomsInProofsByListOfProblems(in LSK, out LSKN)
%%%		LSK: list of problem subkeys
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByListOfProblems(LSK,LSKN):-
		numberOfAxiomsInProofsByListOfProblems(_,LSK,LSKN).

%%% numberOfAxiomsInProofsByListOfProblems(in T, in LSK, out LSKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByListOfProblems(T,LSK,LSKN):-
		findall((SK,N),(member(SK,LSK),numberOfAxiomsInProofsOfProblem(T,SK,N)),LSKN).





%%% numberOfPassedTests(out N)
%%%		N: number of passed tests

numberOfPassedTests(N):-
		findall((P,I),(testInformation(P,_,I,T1,T2),stepAxiom(_,T1,_,_,_),stepAxiom(_,T2,_,_,_)),LI),
		list_to_set(LI,SI),
		length(SI,N).





%%% averageRuntime(out RT)
%%%		RT: the average runtime of solved problems

averageRuntime(RT):-
		findall(RT,proofTime(_,_,RT),LRT),
		length(LRT,N),
		(
		 (
		  N \= 0,
		  !,
		  sum_list(LRT,S),
		  RT is S/N
		 );
		 (
		  RT = 0
		 )
		).

%%% averageRuntimeOfProblem(in SK, out RT)
%%%		SK: problem subkey
%%%		RT: the average runtime of solved problems of problem SK

averageRuntimeOfProblem(SK,RT):-
		findall(RT,proofTime(SK,_,RT),LRT),
		length(LRT,N),
		(
		 (
		  N \= 0,
		  !,
		  sum_list(LRT,S),
		  RT is S/N
		 );
		 (
		  RT = 0
		 )
		).

%%% averageRuntimeByProblem(out LRT)
%%%		LRT: the average runtime of solved problems by problem

averageRuntimeByProblem(LSKRT):-
		getProblemSubkeys(LSK),
		findall((SK,RT),(member(SK,LSK),averageRuntimeOfProblem(SK,RT)),LSKRT).

%%% averageRuntimeOfListOfProblems(in LSK, out RT)
%%%		LSK: list of problem subkeys
%%%		RT: the average runtime of solved problems in LSK

averageRuntimeOfListOfProblems(LSK,RT):-
		findall(RT,(member(SK,LSK),averageRuntimeOfProblem(SK,RT)),LRT),
		length(LRT,N),
		(
		 (
		  N \= 0,
		  !,
		  sum_list(LRT,S),
		  RT is S/N
		 );
		 (
		  RT = 0
		 )
		).

%%% averageRuntimeTruthProblems(out RT)
%%%		LRT: the average runtime of solved truth problems

averageRuntimeTruthProblems(RT):-
		getTruthProblemSubkeys(LSK),
		findall(RT,(member(SK,LSK),proofTime(SK,_,RT)),LRT),
		length(LRT,N),
		(
		 (
		  N \= 0,
		  !,
		  sum_list(LRT,S),
		  RT is S/N
		 );
		 (
		  RT = 0
		 )
		).

%%% averageRuntimeFalsityProblems(out RT)
%%%		LRT: the average runtime of solved falsity problems

averageRuntimeFalsityProblems(RT):-
		getFalsityProblemSubkeys(LSK),
		findall(RT,(member(SK,LSK),proofTime(SK,_,RT)),LRT),
		length(LRT,N),
		(
		 (
		  N \= 0,
		  !,
		  sum_list(LRT,S),
		  RT is S/N
		 );
		 (
		  RT = 0
		 )
		).







%%% mfuAxiomsInProofs(out LAKN)
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofs(LAKN):-
		mfuAxiomsInProofs(_,LAKN).

%%% mfuAxiomsInProofs(in T, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofs(T,NLAKN):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,NLAKN),
		print_mfuAxioms(NLAKN).

%%% topMFUAxiomsInProofs(in M, out LAKN)
%%%		M: number of top items
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofs(M,LAKN):-
		topMFUAxiomsInProofs(_,M,LAKN).

%%% topMFUAxiomsInProofs(in T, in M, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		M: number of top items
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofs(T,M,NLAKN):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,SLAKN),
		length(NLAKN,M),
		append(NLAKN,_,SLAKN),
		print_mfuAxioms(NLAKN).

%%% mfuAxiomsInProofsOfProblem(in SK, out LAKN)
%%%		SK: problem subkey
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofsOfProblem(SK,LAKN):-
		mfuAxiomsInProofsOfProblem(_,SK,LAKN).

%%% mfuAxiomsInProofsOfProblem(in T, in SK, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		SK: problem subkey
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofsOfProblem(T,SK,NLAKN):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T), PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,NLAKN),
		print_mfuAxioms(NLAKN).

%%% topMFUAxiomsInProofsOfProblem(in M, in SK, out LAKN)
%%%		M: number of top items
%%%		SK: problem subkey
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofsOfProblem(M,SK,LAKN):-
		topMFUAxiomsInProofsOfProblem(_,M,SK,LAKN).

%%% topMFUAxiomsInProofsOfProblem(in T, in M, in SK, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		M: number of top items
%%%		SK: problem subkey
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofsOfProblem(T,M,SK,NLAKN):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T), PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,SLAKN),
		length(NLAKN,M),
		append(NLAKN,_,SLAKN),
		print_mfuAxioms(NLAKN).

%%% mfuAxiomsInProofsOfListOfProblems(in LSK, out LAKN)
%%%		LSK: list of problem subkeys
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofsOfListOfProblems(LSK,LAKN):-
		mfuAxiomsInProofsOfListOfProblems(_,LSK,LAKN).

%%% mfuAxiomsInProofsOfListOfProblems(in T, in SK, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

mfuAxiomsInProofsOfListOfProblems(T,LSK,NLAKN):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,NLAKN),
		print_mfuAxioms(NLAKN).

%%% topMFUAxiomsInProofsOfListOfProblems(in M, in SK, out LAKN)
%%%		M: number of top items
%%%		LSK: list of problem subkeys
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofsOfListOfProblems(M,LSK,LAKN):-
		topMFUAxiomsInProofsOfListOfProblems(_,M,LSK,LAKN).

%%% topMFUAxiomsInProofsOfListOfProblems(in T, in M, in SK, out LAKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		M: number of top items
%%%		LSK: list of problem subkeys
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK

topMFUAxiomsInProofsOfListOfProblems(T,M,LSK,NLAKN):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK), LAK),
		list_to_set(LAK,SAK),
		findall((AK,N),(member(AK,SAK),occurrences_of_term(AK,LAK,N)),LAKN),
		sort(2,@>=,LAKN,SLAKN),
		length(NLAKN,M),
		append(NLAKN,_,SLAKN),
		print_mfuAxioms(NLAKN).


%%% getListOfProofsByAxiom(in AK, out LPK)
%%%		AK: axiom key
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiom(AK,NLPK):-
		getListOfProofsByAxiom(_,AK,NLPK).

%%% getListOfProofsByAxiom(in T, in AK, out LPK)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		AK: axiom key
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiom(T,AK,NLPK):-
		findall(PK,stepAxiom(_,PK,_,AK,T),LPK),
		list_to_set(LPK,NLPK).

%%% getListOfProofsByAxiomOfProblem(in AK, in SK, out LPK)
%%%		AK: axiom key
%%%		SK: problem subkey
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomOfProblem(AK,SK,NLPK):-
		getListOfProofsByAxiomOfProblem(_,AK,SK,NLPK).

%%% getListOfProofsByAxiomOfProblem(in T, in AK, in SK, out LPK)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		AK: axiom key
%%%		SK: problem subkey
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomOfProblem(T,AK,SK,NLPK):-
		findall(PK,stepAxiom(SK,PK,_,AK,T),LPK),
		list_to_set(LPK,NLPK).

%%% getListOfProofsByAxiomOfListOfProblems(in AK, in LSK, out LPK)
%%%		AK: axiom key
%%%		LSK: list of problem subkeys
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomOfListOfProblems(AK,LSK,NLPK):-
		getListOfProofsByAxiomOfListOfProblems(_,AK,LSK,NLPK).

%%% getListOfProofsByAxiomOfListOfProblems(in T, in AK, in LSK, out LPK)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		AK: axiom key
%%%		LSK: list of problem subkeys
%%%		LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomOfListOfProblems(T,AK,LSK,NLPK):-
		findall(PK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK)),LPK),
		list_to_set(LPK,NLPK).

%%% getListOfProofsByAxiomByProblem(in AK, out LSKLPK)
%%%		AK: axiom key
%%%		LSKLPK: list of pairs (SK,LPK) where
%%%			SK:	problem subkey
%%%			LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomByProblem(AK,LSKLPK):-
		getListOfProofsByAxiomByProblem(_,AK,LSKLPK).

%%% getListOfProofsByAxiomByProblem(in T, in AK, out LSKLPK)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		AK: axiom key
%%%		LSKLPK: list of pairs (SK,LPK) where
%%%			SK:	problem subkey
%%%			LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomByProblem(T,AK,LSKLPK):-
		getProblemSubkeys(LSK),
		findall((SK,N),(member(SK,LSK),getListOfProofsByAxiomOfProblem(T,AK,SK,N)),LSKLPK).

%%% getListOfProofsByAxiomByListOfProblems(in AK, in LSK, out LSKLPK)
%%%		AK: axiom key
%%%		LSK: list of problem subkeys
%%%		LSKLPK: list of pairs (SK,LPK) where
%%%			SK:	problem subkey
%%%			LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomByListOfProblems(AK,LSK,LSKLPK):-
		getListOfProofsByAxiomByListOfProblems(_,AK,LSK,LSKLPK).

%%% getListOfProofsByAxiomByListOfProblems(in T, in AK, in LSK, out LSKLPK)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		AK: axiom key
%%%		LSK: list of problem subkeys
%%%		LSKLPK: list of pairs (SK,LPK) where
%%%			SK:	problem subkey
%%%			LPK: list of proof keys where the axiom is used

getListOfProofsByAxiomByListOfProblems(T,AK,LSK,LSKLPK):-
		findall((SK,N),(member(SK,LSK),getListOfProofsByAxiomOfProblem(T,AK,SK,N)),LSKLPK).















numberOfUnitClauses(T,N):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK,axiom(_,AK,_,'unitClause')), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfNonAtomicAxioms(T,N):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK,axiom(_,AK,_,'nonAtomicAxiom')), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).



numberOfAxiomsByFile(T,LF,N):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK,axiom(AF,AK,_,_),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfUnitClausesByFile(T,LF,N):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK,axiom(AF,AK,_,'unitClause'),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfNonAtomicAxiomsByFile(T,LF,N):-
		findall(AK,(stepAxiom(_,PK,_,AK,T), PK \= AK,axiom(AF,AK,_,'nonAtomicAxiom'),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).




numberOfUnitClausesOfListOfProblems(T,LSK,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(_,AK,_,'unitClause')), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfNonAtomicAxiomsOfListOfProblems(T,LSK,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(_,AK,_,'nonAtomicAxiom')), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).



numberOfAxiomsOfListOfProblemsByFile(T,LSK,LF,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(AF,AK,_,_),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfUnitClausesOfListOfProblemsByFile(T,LSK,LF,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(AF,AK,_,'unitClause'),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).

numberOfNonAtomicAxiomsOfListOfProblemsByFile(T,LSK,LF,N):-
		findall(AK,(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(AF,AK,_,'nonAtomicAxiom'),memberchk(AF,LF)), LAK),
		list_to_set(LAK,SAK),
		length(SAK,N).





% numberOfProofs
numberOfProofsTT(N):-
		getTruthProblemSubkeys(LSK),
		numberOfProofsOfListOfProblems(_,LSK,N).		
numberOfProofsFT(N):-
		getFalsityProblemSubkeys(LSK),
		numberOfProofsOfListOfProblems(_,LSK,N).		

numberOfAxioms(N):-
		numberOfAxiomsInProofs(N).
numberOfAxiomsTT(N):-
		getTruthProblemSubkeys(LSK),
		numberOfAxiomsInProofsOfListOfProblems(_,LSK,N).
numberOfAxiomsFT(N):-
		getFalsityProblemSubkeys(LSK),
		numberOfAxiomsInProofsOfListOfProblems(_,LSK,N).

numberOfUnitClauses(N):-
		numberOfUnitClauses(_,N).
numberOfUnitClausesTT(N):-
		getTruthProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblems(_,LSK,N).
numberOfUnitClausesFT(N):-
		getFalsityProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblems(_,LSK,N).

numberOfNonAtomicAxioms(N):-
		numberOfNonAtomicAxioms(_,N).
numberOfNonAtomicAxiomsTT(N):-
		getTruthProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblems(_,LSK,N).
numberOfNonAtomicAxiomsFT(N):-
		getFalsityProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblems(_,LSK,N).

numberOfAxiomsPredefinitions(N):-
		LF = ['predefinitions'],
		numberOfAxiomsByFile(_,LF,N).
numberOfAxiomsTTPredefinitions(N):-
		LF = ['predefinitions'],
		getTruthProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfAxiomsFTPredefinitions(N):-
		LF = ['predefinitions'],
		getFalsityProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesPredefinitions(N):-
		LF = ['predefinitions'],
		numberOfUnitClausesByFile(_,LF,N).
numberOfUnitClausesTTPredefinitions(N):-
		LF = ['predefinitions'],
		getTruthProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesFTPredefinitions(N):-
		LF = ['predefinitions'],
		getFalsityProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsPredefinitions(N):-
		LF = ['predefinitions'],
		numberOfNonAtomicAxiomsByFile(_,LF,N).
numberOfNonAtomicAxiomsTTPredefinitions(N):-
		LF = ['predefinitions'],
		getTruthProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsFTPredefinitions(N):-
		LF = ['predefinitions'],
		getFalsityProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).


numberOfAxiomsTopLevel(N):-
		LF = ['merge195'],
		numberOfAxiomsByFile(_,LF,N).
numberOfAxiomsTTTopLevel(N):-
		LF = ['merge195'],
		getTruthProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfAxiomsFTTopLevel(N):-
		LF = ['merge195'],
		getFalsityProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesTopLevel(N):-
		LF = ['merge195'],
		numberOfUnitClausesByFile(_,LF,N).
numberOfUnitClausesTTTopLevel(N):-
		LF = ['merge195'],
		getTruthProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesFTTopLevel(N):-
		LF = ['merge195'],
		getFalsityProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsTopLevel(N):-
		LF = ['merge195'],
		numberOfNonAtomicAxiomsByFile(_,LF,N).
numberOfNonAtomicAxiomsTTTopLevel(N):-
		LF = ['merge195'],
		getTruthProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsFTTopLevel(N):-
		LF = ['merge195'],
		getFalsityProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).


numberOfAxiomsMidLevel(N):-
		LF = ['milo1133'],
		numberOfAxiomsByFile(_,LF,N).
numberOfAxiomsTTMidLevel(N):-
		LF = ['milo1133'],
		getTruthProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfAxiomsFTMidLevel(N):-
		LF = ['milo1133'],
		getFalsityProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesMidLevel(N):-
		LF = ['milo1133'],
		numberOfUnitClausesByFile(_,LF,N).
numberOfUnitClausesTTMidLevel(N):-
		LF = ['milo1133'],
		getTruthProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesFTMidLevel(N):-
		LF = ['milo1133'],
		getFalsityProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsMidLevel(N):-
		LF = ['milo1133'],
		numberOfNonAtomicAxiomsByFile(_,LF,N).
numberOfNonAtomicAxiomsTTMidLevel(N):-
		LF = ['milo1133'],
		getTruthProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsFTMidLevel(N):-
		LF = ['milo1133'],
		getFalsityProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).


numberOfAxiomsFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		numberOfAxiomsByFile(_,LF,N).
numberOfAxiomsTTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getTruthProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfAxiomsFTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getFalsityProblemSubkeys(LSK),
		numberOfAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		numberOfUnitClausesByFile(_,LF,N).
numberOfUnitClausesTTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getTruthProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfUnitClausesFTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getFalsityProblemSubkeys(LSK),
		numberOfUnitClausesOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		numberOfNonAtomicAxiomsByFile(_,LF,N).
numberOfNonAtomicAxiomsTTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getTruthProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).
numberOfNonAtomicAxiomsFTFOTransformation(N):-
		LF = ['schemaBinaryRelation','schemaTernaryRelation','schemaQuaternaryRelation','schemaQuintaryRelation','type'],
		getFalsityProblemSubkeys(LSK),
		numberOfNonAtomicAxiomsOfListOfProblemsByFile(_,LSK,LF,N).






averageAxiomsOfListOfProblemsByFile(T,LSK,LF,AT,N):-
		findall((PK,AK),(stepAxiom(SK,PK,_,AK,T),memberchk(SK,LSK),PK \= AK,axiom(AF,AK,_,AT),memberchk(AF,LF)),LPAK),
		list_to_set(LPAK,SPAK),
%print_to_screen('SPAK: '),
%print(SPAK),
		findall(PK,(stepAxiom(SK,PK,_,_,T),memberchk(SK,LSK)),LPK),
		list_to_set(LPK,SPK),
%print_to_screen('\nSPK: '),
%print(SPK),
		length(SPAK,N1),
print_to_screen('\nN1: '),
print(N1),
		length(SPK,N2),
print_to_screen('\nN2: '),
print(N2),
		N3 is (N1*100) / N2,
		N4 is round(N3),
		N is N4 / 100.



averageAxioms(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageTTAxioms(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageFTAxioms(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).


averageUnitClauses(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageTTUnitClauses(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageFTUnitClauses(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).


averageNonAtomicAxioms(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageTTNonAtomicAxioms(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageFTNonAtomicAxioms(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'predefinitions',
				'merge195',
				'milo1133',
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).


% Predefinitions

averageAxiomsPredefinitions(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageTTAxiomsPredefinitions(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageFTAxiomsPredefinitions(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).


averageUnitClausesPredefinitions(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageTTUnitClausesPredefinitions(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageFTUnitClausesPredefinitions(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).


averageNonAtomicAxiomsPredefinitions(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageTTNonAtomicAxiomsPredefinitions(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageFTNonAtomicAxiomsPredefinitions(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['predefinitions'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).

% Top level

averageAxiomsTopLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageTTAxiomsTopLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageFTAxiomsTopLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).


averageUnitClausesTopLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageTTUnitClausesTopLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageFTUnitClausesTopLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).


averageNonAtomicAxiomsTopLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageTTNonAtomicAxiomsTopLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageFTNonAtomicAxiomsTopLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['merge195'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).

% Mid level

averageAxiomsMidLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageTTAxiomsMidLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageFTAxiomsMidLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).


averageUnitClausesMidLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageTTUnitClausesMidLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageFTUnitClausesMidLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).


averageNonAtomicAxiomsMidLevel(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageTTNonAtomicAxiomsMidLevel(N):-
		getTruthProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageFTNonAtomicAxiomsMidLevel(N):-
		getFalsityProblemSubkeys(LSK),
		LF = ['milo1133'],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).

% FO transformation

averageAxiomsFOTransformation(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageTTAxiomsFOTransformation(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).
averageFTAxiomsFOTransformation(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,_,N).


averageUnitClausesFOTransformation(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageTTUnitClausesFOTransformation(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).
averageFTUnitClausesFOTransformation(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'unitClause',N).


averageNonAtomicAxiomsFOTransformation(N):-
		getTruthProblemSubkeys(TT),
		getFalsityProblemSubkeys(FT),
		append(TT,FT,LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageTTNonAtomicAxiomsFOTransformation(N):-
		getTruthProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).
averageFTNonAtomicAxiomsFOTransformation(N):-
		getFalsityProblemSubkeys(LSK),
		LF = [
				'schemaBinaryRelation',
				'schemaTernaryRelation',
				'schemaQuaternaryRelation',
				'schemaQuintaryRelation',
				'type'
		],
		averageAxiomsOfListOfProblemsByFile(_,LSK,LF,'nonAtomicAxiom',N).














/*

%%% numberOfAxiomsInProofsOfListOfProblems(in LSK, out LSKN)
%%%		LSK: list of problem subkeys
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsOfListOfProblems(LSK,LSKN):-
		numberOfAxiomsInProofsOfListOfProblems(_,LSK,LSKN).

%%% numberOfAxiomsInProofsOfListOfProblems(in T, in LSK, out LSKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSK: list of problem subkeys
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsOfListOfProblems(T,LSK,LSKN):-
		findall((SK,N),(member(SK,LSK),numberOfAxiomsInProofsOfProblem(T,SK,N)),LSKN).




%%% numberOfAxiomsInProofsByProblem(out LSKN)
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByProblem(LSKN):-
		numberOfAxiomsInProofsByProblem(_,LSKN).

%%% numberOfAxiomsInProofsByProblem(in T, out LSKN)
%%%		T: type of proofs
%%%			0	-->		The goal IS used in the proof
%%%			1	-->		The goal is NOT used in the proof
%%%		LSKN: list of pairs (SK,N) where
%%%			SK:	problem subkey
%%%			N: the number of axioms used in proofs of SK

numberOfAxiomsInProofsByProblem(T,LSKN):-
		getProblemSubkeys(LSK),
		findall((SK,N),(member(SK,LSK),numberOfAxiomsInProofsOfProblem(T,SK,N)),LSKN).




*/




%%% getListOfProofFiles(in SK, out L)
%%%		SK: proof subkey
%%%		L: list of SK proof files

getListOfProofFiles(SK,L):-
		getProofDirectory(D),
		getProofFilePrefix(P),
		atom_codes(D,LC1),
		atom_codes(P,LC2),
		atom_codes(SK,LC3),
		atom_codes('*',LC4),
		append([LC1,LC2,LC3,LC4],NLC),
		atom_codes(F,NLC),
		expand_file_name(F,L).

%%% getListOfMinimalProofFiles(out LF)
%%%		LF: list of minimal proof files

getListOfMinimalProofFiles(LF):-
		getMinimalProofDirectory(D),
		getProofFilePrefix(P),
		atom_codes(D,LC1),
		atom_codes(P,LC2),
		atom_codes('*',LC3),
		append([LC1,LC2,LC3],NLC),
		atom_codes(F,NLC),
		expand_file_name(F,LF).

%%% getListOfProofKeys(out LPK)
%%%		LPK: list of proof keys

getListOfProofKeys(NLPK):-
		findall(PK,stepAxiom(_,PK,_,_,_),LPK),
		list_to_set(LPK,NLPK).

%%% parseNonMinimalListOfProofFiles(in LF)
%%%		LF: list of proof files
%%%		Action: proof files are parsed

/*
parseNonMinimalListOfProofFiles([]).
parseNonMinimalListOfProofFiles([F|LF]):-
		deletePreviousInformation,
%print_to_screen(F),
%print_to_screen('\n'),
		open(F,read,S),
		getProofKey(F,PK),
		parseProofFile(PK,S),
		close(S),
		createMinimalProofs([F]),
		parseNonMinimalListOfProofFiles(LF).
*/

parseNonMinimalListOfProofFiles(LF):-
		findall(_,(member(F,LF),open(F,read,S),getProofKey(F,PK),parseProofFile(PK,S),close(S),createMinimalProofs([F])),_).

%%% parseListOfProofFiles(in LF)
%%%		LF: list of proof files
%%%		Action: proof files are parsed

/*
parseListOfProofFiles([]).
parseListOfProofFiles([F|LF]):-
%print_to_screen(F),
%print_to_screen('\n'),
		open(F,read,S),
		getProofKey(F,PK),
		parseProofFile(PK,S),
		close(S),
		parseListOfProofFiles(LF).
*/

parseListOfProofFiles(LF):-
		findall(_,(member(F,LF),open(F,read,S),getProofKey(F,PK),parseProofFile(PK,S),close(S)),_).


%%% getProofKey(in F, out PK)
%%%		F: proof file name
%%%		PK: proof key

getProofKey(F,PK):-
		getProofFilePrefix(P),
		atom_codes(F,LCF),
		atom_codes(P,LCP),
		splitList(LCP,LCF,_,LC),
		atom_codes('.',LCD),
		splitList(LCD,LC,NLC,_),
		atom_codes(PK,NLC).

%%% parseProofFile(in PK, in S)
%%%		PK: proof key
%%%		S: input stream
%%%		Action: the proof file is parsed

parseProofFile(PK,S):-
		isSuccessfulProof(S,B),
		(
		 (
		  B = 1,
		  !,
		  parseSuccessfulProof(PK,S)
		 );
		 (
		  true
		 )
		).

%%% isSuccessfulProof(in S, out B)
%%%		S: input stream
%%%		B:
%%%				1	if the proof IS successful
%%%				0	if the proof is NOT successful

isSuccessfulProof(S,B):-
		read_line_to_codes(S,L),
		(
		 (
		  L = end_of_file,
		  !,
		  B = 0
		 );
		 (
		  L = [],
		  !,
		  isSuccessfulProof(S,B)
		 );
		 (
		  atom_codes('% SZS output start Proof for ',LC),
		  append(LC,_,L),
		  !,
		  B = 1
		 );
		 (
		  isSuccessfulProof(S,B)
		 )
		).
		
%%% parseSuccessfulProof(in PK, in S)
%%%		PK: proof key
%%%		S: input stream
%%%		Action: the proof is parsed

parseSuccessfulProof(PK,S):-
		parseProofStep(PK,S,E),
		(
		 (
		  E = 1,
		  !
		 );
		 (
		  parseSuccessfulProof(PK,S)
		 )
		).

%%% parseProofStep(in PK, in S, out E)
%%%		PK: proof key
%%%		S: input stream
%%%		E:
%%%			E = 1	if the proof IS finished
%%%			E = 0	if the proof is NOT finished
%%%		Action: the next proof step is parsed

parseProofStep(PK,S,E):-
		parseStepKey(S,SK,E),
%print_to_screen(SK),
%print_to_screen('\n'),
		(
		 (
		  E = 1,
		  !
		 );
		 (
		  E = 0,
		  !,
/*
(
 (
  SK = 'f276117',
  !,
  trace
 );
 (
  true
 )
),
*/
		  parseStepFormula(S,SF),
		  parseStepSource(S,SS),
		  saveProofStep(PK,SK,SF,SS)
		 );
		 (
		  E = -1
		 )
		).

%%% parseStepKey(in S, out SK, out E)
%%%		S: input stream
%%%		SK: step key
%%%		E:
%%%			E = 1	if the proof IS finished
%%%			E = 0	if the proof is NOT finished

parseStepKey(S,SK,E):-
		read_line_to_codes(S,L),
		(
		 (
		  atom_codes('% SZS output end Proof for ',LCE),
		  append(LCE,_,L),
		  !,
		  E = 1,
		  SK = ''
		 );
		 (
		  E = 0,
		  atom_codes('fof(',LCF),
		  splitList(LCF,L,_,LC),
		  LC \= [],
		  !,
		  atom_codes(',',LCC),
		  splitList(LCC,LC,NLC,_),
		  atom_codes(SK,NLC)
		 );
		 (
		  E = -1
		 )
		).

%%% parseStepFormula(in S, out SF)
%%%		S: input stream
%%%		SF: formula

parseStepFormula(S,SF):-
		read_line_to_codes(S,L),
		atom_codes('  ',LCB),
		splitList(LCB,L,_,LC),
		atom_codes('),',LCE),
		append(NLC,LCE,LC),
		atom_codes(SF,NLC).

%%% parseStepSource(in S, out SS)
%%%		S: input stream
%%%		SS: step source

parseStepSource(S,SS):-
		read_line_to_codes(S,L),
		atom_codes('  ',LCB),
		splitList(LCB,L,_,LC),
		atom_codes(').',LCP),
		splitList(LCP,LC,NLC,_),
		atom_codes(A,NLC),
		term_to_atom(T,A),
		(
		 (
		  T = file(_,AK),
		  !,
		  SS = axiom(AK)
		 );
		 (
		  T = inference(_,_,LF),
		  !,
		  SS = inference(LF)
		 );
		 (
		  T = introduced(_,_),
		  !,
		  SS = new
		 )
		).

%%% saveProofStep(in PK, in SK, in SF, in SS)
%%%		PK: proof key
%%%		SK: step key
%%%		SF: step formula
%%%		SS: step source
%%%		Action: the proof step is stored using dynamic predicates

saveProofStep(PK,SK,SF,axiom(AK)):-
		!,
		assertz((step(PK,SK,SF,axiom(AK)))),
		extractProofSubkey(PK,PSK),
		assertz((stepAxiom(PSK,PK,SK,AK,-1))).
saveProofStep(PK,SK,SF,inference(LF)):-
		!,
		assertz((step(PK,SK,SF,inference(LF)))).
saveProofStep(PK,SK,SF,new):-
		!,
		assertz((step(PK,SK,SF,new))).

%%% extractProofSubkey(in PK, out PSK)
%%%		PK: proof key
%%%		PSK: proof subkey of PK

extractProofSubkey(PK,PSK):-
		getProblemSubkeys(LSK),
		atom_codes(PK,LCPK),
		findall((N,SK),(member(SK,LSK),atom_codes(SK,LCSK),append(LCSK,_,LCPK),length(LCSK,N)),NSK),
		sort(NSK,SSK),
		last(SSK,(_,PSK)).

%%% classifyProofs(in LPK)
%%%		LPK: list of proof keys
%%%		Action: the type of the proof is updated

classifyProofs([]).
classifyProofs([PK|LPK]):-
		stepAxiom(PSK,PK,_,PK,-1),
		!,
		findall(stepAxiom(PSK,PK,SK,AK,0),stepAxiom(PSK,PK,SK,AK,-1),LA),
		retractall((stepAxiom(_,PK,_,_,_))),
		findall(_,(member(A,LA),assertz((A))),_),
		classifyProofs(LPK).		
classifyProofs([PK|LPK]):-
		findall(stepAxiom(PSK,PK,SK,AK,1),stepAxiom(PSK,PK,SK,AK,-1),LA),
		retractall((stepAxiom(_,PK,_,_,_))),
		findall(_,(member(A,LA),assertz((A))),_),
		classifyProofs(LPK).		

%%% getProofAxioms(in LPK)
%%%		LPK: list of proof keys
%%%		Action: the list of axioms used in the proof is stored

getProofAxioms([]).
getProofAxioms([PK|LPK]):-
		findall(AK,stepAxiom(_,PK,_,AK,_),LAK),
		list_to_set(LAK,SAK),
		length(SAK,N),
		assertz((proofAxioms(PK,SAK,N))),
		getProofAxioms(LPK).

%%% saveMinimalProofs
%%%		Action: information about minimal proofs is stored

saveMinimalProofs:-
		getProofsTime,
		getMinimalProofDirectory(D),
		getMinimalProofFile(F),
		atom_concat(D,F,NF),
		open(NF,write,S),
		findall(_,(stepAxiom(PSK,PK,SK,AK,T),print(S,stepAxiom(PSK,PK,SK,AK,T)),print_to_file(S,'\n')),_),
		findall(_,(step(PK,SK,SF,AS),print(S,step(PK,SK,SF,AS)),print_to_file(S,'\n')),_),
		findall(_,(proofAxioms(PK,LAK,N),print(S,proofAxioms(PK,LAK,N)),print_to_file(S,'\n')),_),
		findall(_,(proofTime(PK,SK,T),print(S,proofTime(PK,SK,T)),print_to_file(S,'\n')),_),
		close(S).

%%% loadMinimalProofInformation(in S)
%%%		S: input stream
%%%		Action: information about minimal proofs is loaded

loadMinimalProofInformation(S):-
		read_line_to_codes(S,L),
		(
		 (
		  L = end_of_file,
		  !
		 );
		 (
		  L = [],
		  !,
		  loadMinimalProofInformation(S)
		 );
		 (
		  atom_codes(A,L),
		  term_to_atom(T,A),
		  assertz((T)),
		  loadMinimalProofInformation(S)
		 )
		).

%%% saveProofTime(in SK, in PK, in OF)
%%%		SK: proof subkey
%%%		PK: proof key
%%%		OF: output file
%%%		Action: the proof time of successful proofs is stored

saveProofTime(SK,PK,OF):-
		open(OF,read,S),
		isSuccessfulProof(S,B),
		(
		 (
		  B = 1,
		  !,
		  parsePreciseProofTime(S,T),
		  (
		   (
		    T = 3600,
		    !,
		    true
		   );
		   (
		    assertz((proofTime(SK,PK,T)))
		   )
		  )
		 );
		 (
		  true
		 )
		),
		close(S).

%%% showProofTree(in PK)
%%%		PK: proof key
%%%		Action: the proof is prompted in tree form

showProofTree(PK):-
		current_output(S),
		step(PK,SK,'$false',SS),
		showProofTreeStep(S,PK,SK,'$false',SS,[],0),
		showProofTime(S,PK).

%%% showProofTree(in S, in PK)
%%%		S: output stream
%%%		PK: proof key
%%%		Action: the proof is prompted in tree form

showProofTree(S,PK):-
		step(PK,SK,'$false',SS),
		showProofTreeStep(S,PK,SK,'$false',SS,[],0),
		showProofTime(S,PK).

%%% showProofTreeStep(in S, in PK, in SK, in SF, in SS, in P, in T)
%%%		S: output stream
%%%		PK: proof key
%%%		SK: step key
%%%		SF: step formula
%%%		SS: step source
%%%		P: step prefix
%%%		T: tab number

showProofTreeStep(S,_,_,SF,axiom(AK),P,T):-
		!,
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,': '),
		print_to_file(S,AK),
		print_to_file(S,' --> '),
		print_to_file(S,SF),
		print_to_file(S,'\n').
showProofTreeStep(S,_,_,SF,new,P,T):-
		!,
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,': '),
		print_to_file(S,new),
		print_to_file(S,' --> '),
		print_to_file(S,SF),
		print_to_file(S,'\n').
showProofTreeStep(S,PK,_,_,inference([NSK]),P,T):-
		!,
		step(PK,NSK,NSF,NSS),
		showProofTreeStep(S,PK,NSK,NSF,NSS,P,T).
showProofTreeStep(S,PK,_,SF,inference(LF),P,T):-
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,' -- > '),
		print_to_file(S,SF),
		print_to_file(S,'\n'),
		NT is T+1,
		findall(_,(nth1(I,LF,F),append(P,[I],NP),step(PK,F,NSF,SS),showProofTreeStep(S,PK,F,NSF,SS,NP,NT)),_).

%%% showProofSketch(in D, in PK)
%%%		D: degree of the sketch
%%%		PK: proof key
%%%		Action: the sketch of the proof is prompted

showProofSketch(D,PK):-
		current_output(S),
		step(PK,SK,'$false',SS),
		showProofSketchStep(S,D,PK,SK,'$false',SS,[],0),
		showProofTime(S,PK).

%%% showProofSketch(in S, in D, in PK)
%%%		S: output stream
%%%		D: degree of the sketch
%%%		PK: proof key
%%%		Action: the proof is prompted in tree form

showProofSketch(S,D,PK):-
		step(PK,SK,'$false',SS),
		showProofSketchStep(S,D,PK,SK,'$false',SS,[],0),
		showProofTime(S,PK).

%%% showProofSketchStep(in S, in D, in PK, in SK, in SF, in SS, in P, in T)
%%%		S: output stream
%%%		D: degree of the sketch
%%%		PK: proof key
%%%		SK: step key
%%%		SF: step formula
%%%		SS: step source
%%%		P: step prefix
%%%		T: tab number

showProofSketchStep(S,_,_,_,SF,axiom(AK),P,T):-
		!,
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,': '),
		print_to_file(S,AK),
		print_to_file(S,' --> '),
		print_to_file(S,SF),
		print_to_file(S,'\n').
showProofSketchStep(S,_,_,_,SF,new,P,T):-
		!,
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,': '),
		print_to_file(S,new),
		print_to_file(S,' --> '),
		print_to_file(S,SF),
		print_to_file(S,'\n').
showProofSketchStep(S,D,PK,_,_,inference([NSK]),P,T):-
		!,
		step(PK,NSK,NSF,NSS),
		showProofSketchStep(S,D,PK,NSK,NSF,NSS,P,T).
showProofSketchStep(S,0,PK,_,SF,inference(LF),P,T):-
		!,
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,' -- > '),
		print_to_file(S,SF),
		print_to_file(S,'\n'),
		NT is T+1,
		findall(A,(member(F,LF),getProofSketchStepAxioms(PK,F,A)),LA),
		append(LA,NLA),
		list_to_set(NLA,NSA),
		findall(_,(nth1(I,NSA,A),append(P,[I],NP),step(PK,A,NSF,SS),showProofSketchStep(S,0,PK,A,NSF,SS,NP,NT)),_).
showProofSketchStep(S,D,PK,_,SF,inference(LF),P,T):-
		print_tabs(S,T),
		print_to_file(S,P),
		print_to_file(S,' -- > '),
		print_to_file(S,SF),
		print_to_file(S,'\n'),
		NT is T+1,
		ND is D-1,
		findall(_,(nth1(I,LF,F),append(P,[I],NP),step(PK,F,NSF,SS),showProofSketchStep(S,ND,PK,F,NSF,SS,NP,NT)),_).

%%% showProofTime(in S, in PK)
%%%		S: output stream
%%%		PK: proof key
%%%		Action: the time proof is prompted

showProofTime(S,PK):-
		proofTime(_,PK,T),
		print_to_file(S,'Time: '),
		print_to_file(S,T),
		print_to_file(S,' seconds\n').

%%% getProofSketchStepAxioms(in PK, in SK, out LSK)
%%%		PK: proof key
%%%		SK: step key
%%%		LSK: list of step keys with axioms that yields SK

getProofSketchStepAxioms(PK,SK,[SK]):-
		step(PK,SK,_,axiom(_)),
		!.
getProofSketchStepAxioms(PK,SK,[]):-
		step(PK,SK,_,new),
		!.
getProofSketchStepAxioms(PK,SK,NLSK):-
		step(PK,SK,_,inference(LF)),
		findall(NSK,(member(F,LF),getProofSketchStepAxioms(PK,F,NSK)),LSK),
		append(LSK,NLSK).

%%% createMinimalProofs(in LF)
%%%		LF: list of proof files
%%%		Action: minimal proofs are obtained

createMinimalProofs([]).
createMinimalProofs([F|LF]):-
		getProofKey(F,PK),
		step(PK,_,_,_),
		!,
		toMinimalProof(PK),
		createMinimalProofs(LF).
createMinimalProofs([_|LF]):-
		createMinimalProofs(LF).

%%% toMinimalProof(in PK)
%%%		PK: proof key
%%%		Action: the equivalent minimal proof of PK is created

toMinimalProof(PK):-
		findall(SK,step(PK,SK,_,axiom(_)),LSK),
		list_to_set(LSK,SSK),
		getMinimalProofTime(PK,SSK,T),
		removeSuperflousProofSteps(T,PK,[],SSK,NSSK),
		checkMinimalProof(T,PK,NSSK).

%%% getMinimalProofTime(in PK, in LSK, out T)
%%%		PK: problem key
%%%		LSK: list of step keys
%%%		T: proof time

getMinimalProofTime(PK,LSK,NT):-
		writeProblem(PK,LSK,PF),
		executeProblem(36000,PK,PF,OF),
		getProofTime(OF,T),
		(
		 (
		  T < 5,
		  !,
		  NT = 5
		 );
		 (
		  NT = T
		 )
		).

%%% getProofTime(in OF, out T)
%%%		OF: output file
%%%		T: proof time

getProofTime(OF,T):-
		open(OF,read,S),
		isSuccessfulProof(S,B),
		(
		 (
		  B = 1,
		  !,
		  parseProofTime(S,T)
		 );
		 (
		  T = 3600
		 )
		),
		close(S).

%%% parseProofTime(in S, out T)
%%%		S: output stream
%%%		T: proof time

parseProofTime(S,NT):-
		read_line_to_codes(S,L),
		(
		 (
		  L = end_of_file,
		  !,
		  NT = 3600
		 );
		 (
		  L = [],
		  !,
		  parseProofTime(S,NT)
		 );
		 (
		  atom_codes('% Success in time ',LCP),
		  append(LCP,LC1,L),
		  atom_codes(' s',LCS),
		  append(LCT,LCS,LC1),
		  !,
		  term_to_atom(ST,LCT),
		  T1 is ceiling(ST),
		  T2 is T1*1.5,
		  NT is ceiling(T2)
		 );
		 (
		  parseProofTime(S,NT)
		 )
		).

%%% parsePreciseProofTime(in S, out T)
%%%		S: output stream
%%%		T: proof time

parsePreciseProofTime(S,NT):-
		read_line_to_codes(S,L),
		(
		 (
		  L = end_of_file,
		  !,
		  NT = 3600
		 );
		 (
		  L = [],
		  !,
		  parsePreciseProofTime(S,NT)
		 );
		 (
		  atom_codes('% Success in time ',LCP),
		  append(LCP,LC1,L),
		  atom_codes(' s',LCS),
		  append(LCT,LCS,LC1),
		  !,
		  term_to_atom(NT,LCT)
		 );
		 (
		  parsePreciseProofTime(S,NT)
		 )
		).

%%% removeSuperflousProofSteps(in T, in PK, in LSK1, in LSK2, out NLSK)
%%%		T: proof time
%%%		PK: proof key
%%%		LSK1: list of non-superflous step keys
%%%		LSK2: list of step keys to be checked
%%%		NLSK: list of non-superflous step keys that is obtained from LSK1 and LSK2

removeSuperflousProofSteps(_,_,LSK,[],LSK).
removeSuperflousProofSteps(T,PK,LSK1,[SK|LSK2],NLSK):-
		append(LSK1,LSK2,LSK3),
		isProvable(T,PK,LSK3,B),
%print_to_screen(PK),
%print_to_screen(','),
%print_to_screen(SK),
%print_to_screen(','),
%print_to_screen(B),
%print_to_screen('\n'),
		(
		 (
		  B = 1,
		  !,
		  NLSK1 = LSK1
		 );
		 (
		  append(LSK1,[SK],NLSK1)
		 )
		),
		removeSuperflousProofSteps(T,PK,NLSK1,LSK2,NLSK).

%%% isProvable(in T, in PK, in LSK, out B)
%%%		T: proof time
%%%		PK: proof key
%%%		LSK: list of step keys
%%%		B:
%%%			1	if the problem IS provable
%%%			0	if the problem is NOT provable

isProvable(T,PK,LSK,B):-
		writeProblem(PK,LSK,PF),
		executeProblem(T,PK,PF,OF),
		open(OF,read,S),
		isSuccessfulProof(S,B),
		close(S).

%%% writeProblem(in PK, in LSK, out PF)
%%%		PK: proof key
%%%		LSK: list of step keys
%%%		PF: problem file name
%%%		Action: the problem consisting of the axioms in LSK is written in PF

writeProblem(PK,LSK,PF):-
		getMinimalProofDirectory(D),
		getProblemFileSuffix(N),
		atom_concat(D,PK,F),
		atom_concat(F,N,PF),
		open(PF,write,S),
		findall(_,(member(SK,LSK),writeProblemAxiom(S,PK,SK)),_),
		close(S).

%%% writeProblemAxiom(in S, in PK, in SK)
%%%		S: output stream
%%%		PK: proof key
%%%		SK: step key
%%%		Action: the axiom in SK is written in the output stream

writeProblemAxiom(S,PK,SK):-
		step(PK,SK,F,axiom(PK)),
		!,
		print_to_file(S,'fof('),
		print_to_file(S,PK),
		print_to_file(S,',conjecture,\n\t( '),
		print_to_file(S,F),
		print_to_file(S,' )\n).\n\n').
writeProblemAxiom(S,PK,SK):-
		step(PK,SK,F,axiom(AK)),
		print_to_file(S,'fof('),
		print_to_file(S,AK),
		print_to_file(S,',axiom,\n\t( '),
		print_to_file(S,F),
		print_to_file(S,' )\n).\n\n').

%%% executeProblem(in T, in PK, in PF, out OF)
%%%		T: proof time
%%%		PK: proof key
%%%		PF: problem file
%%%		OF: output file
%%%		Action: the problem in PF is executed and the output is directed to OF

executeProblem(T,PK,PF,OF):-
		getMinimalProofDirectory(D),
		getProofFilePrefix(P),
		getProofFileSuffix(N),
		atom_concat(D,P,NP),
		atom_concat(NP,PK,F),
		atom_concat(F,N,OF),
		fork(PID),
		(
		 (
		  PID = child,
		  !,
		  exec('./vampire.sh'(T,PF,OF))
		 );
		 (
		  wait(PID,_)
		 )
		).

%%% checkMinimalProof(in T, in PK, in LSK)
%%%		T: proof time
%%%		PK: proof key
%%%		LSK: list of step keys

checkMinimalProof(T,PK,LSK):-
		isProvable(T,PK,LSK,B),
		(
		 (
		  B = 1,
		  !
		 );
		 (
		  print_to_screen('\nWarning!!! --> '),
		  print_to_screen(PK),
		  print_to_screen(' minimal problem is not provable!')
		 )
		).

%%% splitList(in S, in L, out B, out A)
%%%		S: split criteria
%%%		L: list
%%%		B: list occurring before S in L
%%%		A: list occurring after S in L

splitList(_,[],[],[]):-
		!.
splitList(S,L,B,A):-
		append(S,A,L),
		!,
		B = [].
splitList(S,[E|L],[E|B],A):-
		splitList(S,L,B,A).

%%% print_mfuAxioms(in LAKN)
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK
%%%		Action: the list of most frequently used axioms is prompted

print_mfuAxioms(LAKN):-
		current_output(S),
		writeListOfAxiomsAndOccurrences(S,LAKN).

%%% writeListOfAxiomsAndOccurrences(in S, in LAKN)
%%%		S: output stream
%%%		LAKN: list of pairs (AK,N) where
%%%				AK	is an axiom key
%%%				N	is the number of occurrences of AK
%%%		Action: the list of axioms and its occurrences is prompted

writeListOfAxiomsAndOccurrences(_,[]).
writeListOfAxiomsAndOccurrences(S,[(AK,N)|LAKN]):-
		print_to_file(S,'\n* '),
		print_to_file(S,AK),
		print_to_file(S,' ['),
		print_to_file(S,N),
		print_to_file(S,'] --> '),
		step(_,_,F,axiom(AK)),
		!,
		print_to_file(S,F),
		print_to_file(S,'\n'),
		writeListOfAxiomsAndOccurrences(S,LAKN).

%%% print_tabs(in S, in T)
%%%		S: output stream
%%%		T: tab number
%%%		Action: tabs are prompted

print_tabs(S,T):-
		length(L,T),
		findall(_,(member(_,L),print_to_file(S,'\t')),_).

%%% print_to_file(in S, in T)
%%%		S: output strem
%%%		T: term
%%%		Action: the term is written in S

print_to_file(S,T):-
		write_term(S,T,[]).

%%% print_to_screen(in T)
%%%		T: term
%%%		Action: the term is prompted

print_to_screen(T):-
		current_output(S),
		print_to_file(S,T).
